
# line 2 "parse.y"
#include <stdio.h>
#include <ctype.h>
#include <Xm/Xm.h>
#include "../util/DialogF.h"
#include "../histo_util/histo.h"
#include "interpret.h"
#include "variablePanel.h"
#include "parse.h"

typedef Symbol *SymPtr;
#define YYSTYPE SymPtr

/* Macros to add error processing to AddOp and AddSym calls */
#define ADD_OP(op) if (!AddOp(op, &ErrMsg)) return 1
#define ADD_SYM(sym) if (!AddSym(sym, &ErrMsg)) return 1
# define NUMBER 257
# define VAR 258
# define BLTIN 259
# define UNDEF 260
# define OR 261
# define AND 262
# define GT 263
# define GE 264
# define LT 265
# define LE 266
# define EQ 267
# define NE 268
# define UNARYMINUS 269
# define NOT 270
# define POW 271

#include <stdlib.h>
/* #include <malloc.h> */
/* #include <memory.h> */
/* #include <unistd.h> */
/* #include <values.h> */

#ifdef __cplusplus

#ifndef yyerror
	void yyerror(const char *);
#endif
#ifndef yylex
	extern "C" int yylex(void);
#endif
	int yyparse(void);

#endif
#define yyclearin yychar = -1
#define yyerrok yyerrflag = 0
extern int yychar;
extern int yyerrflag;
#ifndef YYSTYPE
#define YYSTYPE int
#endif
YYSTYPE yylval;
YYSTYPE yyval;
typedef int yytabelem;
#ifndef YYMAXDEPTH
#define YYMAXDEPTH 150
#endif
#if YYMAXDEPTH > 0
int yy_yys[YYMAXDEPTH], *yys = yy_yys;
YYSTYPE yy_yyv[YYMAXDEPTH], *yyv = yy_yyv;
#else	/* user does initial allocation */
int *yys;
YYSTYPE *yyv;
#endif
static int yymaxdepth = YYMAXDEPTH;
# define YYERRCODE 256

# line 66 "parse.y"
 /* User Subroutines Section */

/* Max. number of undefined symbols to print in the dialog */
#define MAX_UNDEFINED 15

static int follow(char expect, int ifyes, int ifno);

char *ErrMsg;
char *InputString;
char *InPtr;

/*
** Parse a null terminated string and create a program from it (this is the
** parser entry point).  The program created by this routine can be
** executed using ExecuteProgram.  Returns program on success, or NULL
** on failure.  If the command failed, the error message is returned
** as a pointer to a static string in msg, and the length of the string up
** to where parsing failed in failedAt.
*/
Program *ParseExpr(char *expr, char **msg, int *failedAt)
{
    Program *prog;
    Symbol *s, *toCheckList;
    
    BeginCreatingProgram();
    
    /* call yyparse to parse the string and check for success.  If the parse
       failed, return the error message and string index (the grammar aborts
       parsing at the first error) */
    InputString = expr;
    InPtr = InputString;
    if (yyparse()) {
    	*msg = ErrMsg;
    	*failedAt = InPtr - InputString;
    	FreeProgram(FinishCreatingProgram());
    	return NULL;
    }
    
    /* check that everything was parsed */
    if (*--InPtr != '\0') {
    	*msg = "Syntax Error";
    	*failedAt = InPtr - InputString;
    	FreeProgram(FinishCreatingProgram());
    	return NULL;
    }
    
    /* get the newly created program */
    prog = FinishCreatingProgram();
    
    /* move the symbols which are now defined out of the external symbol
       table and in to the local symbol table */
    toCheckList = prog->externSymList;
    prog->externSymList = NULL;
    while (toCheckList != NULL) {
    	s = toCheckList;
    	toCheckList = s->next;
    	if (s->type == DEFINED) {
    	    s->next = prog->localSymList;
    	    prog->localSymList = s;
    	} else {
    	    s->next = prog->externSymList;
    	    prog->externSymList = s;
    	}
    }
    
    /* parse succeeded */
    *msg = "";
    *failedAt = 0;
    return prog;
}


/*
** Resolve ntuple and user defined variable names in the program "prog".
*/
void ResolveVariableReferences(Program *prog, hsNTuple *ntuple,
	ntupleExtension *ntExt)
{
    Symbol *s;
    int i;
    
    /* Look thru all of the symbols in the symbol table, and try to resolve
       the undefined ones to ntuple variable names.  When found, change their
       symbol type for the interpreter to NT_VAR or USER_DEF_VAR, and
       record their variable index in the symbol */
    for (s=prog->externSymList; s!=NULL; s=s->next) {
	s->type = UNDEFINED;
	for (i=0; i<ntuple->nVariables; i++) {
	    if (!strcmp(s->name, ntuple->names[i])) {
	    	s->type = NT_VAR;
	    	s->index = i;
	    }
	}
	if (ntExt != NULL) {
	    for (i=0; i<ntExt->nVars; i++) {
		if (!strcmp(s->name, ntExt->vars[i].name)) {
	    	    s->type = USER_DEF_VAR;
	    	    s->index = i;
		}
    	    }
    	}
    }
}

/*
** Check for remaining undefined symbols in program prog,
** and warn the user with a nice dialog if there are any.
*/
int WarnUndefined(Program *prog, Widget parent)
{
    Symbol *s;
    char *undefinedSyms[MAX_UNDEFINED], *undefinedString;
    int nUndefined = 0;
    int i, nWritten, len = 0, pos = 0;
    
    /* Look thru all of the symbols in the symbol table, and count the
       undefined ones.  Also record a list of the first MAX_UNDEFINED ones
       to report to the user in a dialog */
    for (s = prog->externSymList; s != NULL; s = s->next) {
    	if (s->type == UNDEFINED) {
    	    if (nUndefined < MAX_UNDEFINED)
    		undefinedSyms[nUndefined] = s->name;
    	    nUndefined++;
    	}
    }
    
    /* Report the undefined symbols to the user */
    if (nUndefined > 0) {
        for (i=0; i<nUndefined && i<MAX_UNDEFINED; i++)
            len += strlen(undefinedSyms[i]) + 1;
        undefinedString = (char *)XtMalloc(sizeof(char)*(len + 7));
        for (i=0; i<nUndefined && i<MAX_UNDEFINED; i++) {
            sprintf(&undefinedString[pos], "%s\n%n", undefinedSyms[i],
            	    &nWritten);
            pos += nWritten;
        }
        if (nUndefined > MAX_UNDEFINED) {
            sprintf(&undefinedString[pos], ".\n.\n.\n%n", &nWritten);
            pos += nWritten;
        }
        undefinedString[pos-1] = '\0'; /* remove trailing newline */
    	DialogF(DF_WARN, parent, 1,
    	    	"%d undefined symbol(s) in expression:\n%s",
    	    	"Acknowledged", nUndefined, undefinedString);
	XtFree(undefinedString);
	return False;
    }
    return True;
}

yylex()
{
    char c;
    int len;
    
    /* skip whitespace */
    while ((c = *InPtr++) == ' ' || c == '\t');
    
    /* return end of input at the end of the string */
    if (c == '\0')
	return 0;
    
    /* process number tokens */
    if (c == '.' || isdigit(c))  { /* number */
        double d;
        InPtr--;
        sscanf(InPtr, "%lf%n", &d, &len);
        InPtr += len;
        yylval = InstallSymbol("", CONSTANT, d);
        return NUMBER;
    }
    
    /* process symbol tokens */
    if (isalpha(c)) {
        Symbol *s;
        char sbuf[100], *p = sbuf;
        do {
	    if (p < sbuf + sizeof(sbuf) -1 ) /* truncate if too long */
		*p++ = c;
	} while ((c = *InPtr++) != EOF && isalnum(c));
	InPtr--;
	*p = '\0';
	if ((s=LookupSymbol(sbuf)) == 0)
            s = InstallSymbol(sbuf, UNDEFINED, 0.0);
	yylval = s;
        if (s->type == FUNCTION)
            return BLTIN;
        else
            return VAR;
    }
    
    /* process quoted string symbols */
    if (c == '\"') {
        Symbol *s;
        char sbuf[100], *p = sbuf;
        while ((c = *InPtr++) != EOF && c != '\"') {
	    if (p < sbuf + sizeof(sbuf) -1) /* truncate if too long */
		*p++ = c;
	}
	*p = '\0';
	if ((s=LookupSymbol(sbuf)) == 0)
            s = InstallSymbol(sbuf, UNDEFINED, 0.0);
	yylval = s;
        return VAR;
    }
    
    /* process remaining two character tokens or return single char as token */
    switch(c) {
    case '>':	return follow('=', GE, GT);
    case '<':	return follow('=', LE, LT);
    case '=':	return follow('=', EQ, '=');
    case '!':	return follow('=', NE, NOT);
    case '|':	return follow('|', OR, '|');
    case '&':	return follow('&', AND, '&');
    case '*':	return follow('*', POW, '*');
    case '^':	return POW;
    default:	return c;
    }
}

/*
** look ahead for >=, etc.
*/
static int follow(char expect, int ifyes, int ifno)
{
    if (*InPtr++ == expect)
	return ifyes;
    InPtr--;
    return ifno;
}

/*
** Called by yacc to report errors (just stores for returning when
** parsing is aborted.  The error token action is to immediate abort
** parsing, so this message is immediately reported to the caller
** of ParseExpr)
*/
yyerror(char *s)
{
    ErrMsg = s;
}
yytabelem yyexca[] ={
-1, 0,
	257, 3,
	258, 3,
	259, 3,
	45, 3,
	270, 3,
	10, 3,
	40, 3,
	-2, 0,
-1, 1,
	0, -1,
	-2, 0,
	};
# define YYNPROD 29
# define YYLAST 292
yytabelem yyact[]={

    51,    16,    14,    18,    15,     3,    17,    28,    29,    34,
    13,    50,    16,    14,    27,    15,    31,    17,     2,     6,
     1,     0,    16,    16,    14,     5,    15,    17,    17,     0,
     0,     0,     0,    16,    14,     0,    15,     0,    17,     0,
     0,     0,    16,    14,     0,    15,     0,    17,    16,    14,
     0,    15,     4,    17,     0,    10,     0,     0,     0,     0,
    11,    10,     0,    30,    32,    33,    11,    35,    36,    37,
    38,    39,    40,    41,    42,    43,    44,    45,    46,    47,
     0,    48,    49,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
    26,    25,    19,    20,    21,    22,    23,    24,     0,     0,
    18,    26,    25,    19,    20,    21,    22,    23,    24,     0,
     0,    18,    26,    25,    19,    20,    21,    22,    23,    24,
     0,    18,    18,    25,    19,    20,    21,    22,    23,    24,
     0,     0,    18,    19,    20,    21,    22,    23,    24,     0,
     0,    18,     7,     8,     9,     0,     0,    18,     7,     8,
     9,     0,     0,     0,     0,    12,     0,     0,     0,     0,
     0,    12 };
yytabelem yypact[]={

  -251,-10000000,    15,-10000000,   -19,-10000000,     4,-10000000,   -54,   -32,
    21,    21,    21,    -1,    21,    21,    21,    21,    21,    21,
    21,    21,    21,    21,    21,    21,    21,-10000000,    21,    21,
   -30,-10000000,  -268,  -268,-10000000,   -20,   -20,  -268,  -268,  -268,
     6,     6,     6,     6,     6,     6,     0,    -9,   -19,   -41,
-10000000,-10000000 };
yytabelem yypgo[]={

     0,    20,    18,    52,    10,    16 };
yytabelem yyr1[]={

     0,     1,     1,     2,     2,     2,     5,     3,     3,     3,
     3,     3,     3,     3,     3,     3,     3,     3,     3,     3,
     3,     3,     3,     3,     3,     3,     3,     4,     4 };
yytabelem yyr2[]={

     0,     7,     3,     0,     4,     7,     7,     3,     3,     2,
     9,     6,     7,     7,     7,     7,     7,     5,     7,     7,
     7,     7,     7,     7,     7,     7,     5,     0,     4 };
yytabelem yychk[]={

-10000000,    -1,    -2,   256,    -3,    10,    -5,   257,   258,   259,
    40,    45,   270,    -4,    43,    45,    42,    47,   271,   263,
   264,   265,   266,   267,   268,   262,   261,    10,    61,    40,
    -3,    -5,    -3,    -3,    10,    -3,    -3,    -3,    -3,    -3,
    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,    -3,
    41,    41 };
yytabelem yydef[]={

    -2,    -2,     0,     2,    27,     4,     9,     7,     8,     0,
     0,     0,     0,     1,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     5,     0,     0,
     0,     9,    17,    26,    28,    12,    13,    14,    15,    16,
    18,    19,    20,    21,    22,    23,    24,    25,     6,     0,
    11,    10 };
typedef struct
#ifdef __cplusplus
	yytoktype
#endif
{ char *t_name; int t_val; } yytoktype;
#ifndef YYDEBUG
#	define YYDEBUG	0	/* don't allow debugging */
#endif

#if YYDEBUG

yytoktype yytoks[] =
{
	"NUMBER",	257,
	"VAR",	258,
	"BLTIN",	259,
	"UNDEF",	260,
	"=",	61,
	"OR",	261,
	"AND",	262,
	"GT",	263,
	"GE",	264,
	"LT",	265,
	"LE",	266,
	"EQ",	267,
	"NE",	268,
	"+",	43,
	"-",	45,
	"*",	42,
	"/",	47,
	"UNARYMINUS",	269,
	"NOT",	270,
	"POW",	271,
	"-unknown-",	-1	/* ends search */
};

char * yyreds[] =
{
	"-no such reduction-",
	"program : asgns expr empty",
	"program : error",
	"asgns : /* empty */",
	"asgns : asgns '\n'",
	"asgns : asgns asgn '\n'",
	"asgn : VAR '=' expr",
	"expr : NUMBER",
	"expr : VAR",
	"expr : asgn",
	"expr : BLTIN '(' expr ')'",
	"expr : '(' expr ')'",
	"expr : expr '+' expr",
	"expr : expr '-' expr",
	"expr : expr '*' expr",
	"expr : expr '/' expr",
	"expr : expr POW expr",
	"expr : '-' expr",
	"expr : expr GT expr",
	"expr : expr GE expr",
	"expr : expr LT expr",
	"expr : expr LE expr",
	"expr : expr EQ expr",
	"expr : expr NE expr",
	"expr : expr AND expr",
	"expr : expr OR expr",
	"expr : NOT expr",
	"empty : /* empty */",
	"empty : empty '\n'",
};
#endif /* YYDEBUG */
/* 
 *	Copyright 1987 Silicon Graphics, Inc. - All Rights Reserved
 */

/* #ident	"@(#)yacc:yaccpar	1.10" */
/* #ident	"$Revision: 1.9 $" */

/*
** Skeleton parser driver for yacc output
*/

/*
** yacc user known macros and defines
*/
#define YYERROR		goto yyerrlab
#define YYACCEPT	return(0)
#define YYABORT		return(1)
#ifdef __cplusplus
#define YYBACKUP( newtoken, newvalue )\
{\
	if ( yychar >= 0 || ( yyr2[ yytmp ] >> 1 ) != 1 )\
	{\
		yyerror( "syntax error - cannot backup" );\
		goto yyerrlab;\
	}\
	yychar = newtoken;\
	yystate = *yyps;\
	yylval = newvalue;\
	goto yynewstate;\
}
#else
#define YYBACKUP( newtoken, newvalue )\
{\
	if ( yychar >= 0 || ( yyr2[ yytmp ] >> 1 ) != 1 )\
	{\
		yyerror( "Syntax error - cannot backup" );\
		goto yyerrlab;\
	}\
	yychar = newtoken;\
	yystate = *yyps;\
	yylval = newvalue;\
	goto yynewstate;\
}
#endif
#define YYRECOVERING()	(!!yyerrflag)
#define YYNEW(type)	malloc(sizeof(type) * yynewmax)
#define YYCOPY(to, from, type) \
	(type *) memcpy(to, (char *) from, yynewmax * sizeof(type))
#define YYENLARGE( from, type) \
	(type *) realloc((char *) from, yynewmax * sizeof(type))
#ifndef YYDEBUG
#	define YYDEBUG	1	/* make debugging available */
#endif

/*
** user known globals
*/
int yydebug;			/* set to 1 to get debugging */

/*
** driver internal defines
*/
#define YYFLAG		(-10000000)

/*
** global variables used by the parser
*/
YYSTYPE *yypv;			/* top of value stack */
int *yyps;			/* top of state stack */

int yystate;			/* current state */
int yytmp;			/* extra var (lasts between blocks) */

int yynerrs;			/* number of errors */
int yyerrflag;			/* error recovery flag */
int yychar;			/* current input token number */



/*
** yyparse - return 0 if worked, 1 if syntax error not recovered from
*/
#if defined(__STDC__) || defined(__cplusplus)
int yyparse(void)
#else
int yyparse()
#endif
{
	register YYSTYPE *yypvt;	/* top of value stack for $vars */

	/*
	** Initialize externals - yyparse may be called more than once
	*/
	yypv = &yyv[-1];
	yyps = &yys[-1];
	yystate = 0;
	yytmp = 0;
	yynerrs = 0;
	yyerrflag = 0;
	yychar = -1;

#if YYMAXDEPTH <= 0
	if (yymaxdepth <= 0)
	{
		if ((yymaxdepth = YYEXPAND(0)) <= 0)
		{
#ifdef __cplusplus
			yyerror( "yacc initialization error" );
#else
			yyerror("Yacc initialization error");
#endif
			YYABORT;
		}
	}
#endif

	goto yystack;
	{
		register YYSTYPE *yy_pv;	/* top of value stack */
		register int *yy_ps;		/* top of state stack */
		register int yy_state;		/* current state */
		register int  yy_n;		/* internal state number info */

		/*
		** get globals into registers.
		** branch to here only if YYBACKUP was called.
		*/
	yynewstate:
		yy_pv = yypv;
		yy_ps = yyps;
		yy_state = yystate;
		goto yy_newstate;

		/*
		** get globals into registers.
		** either we just started, or we just finished a reduction
		*/
	yystack:
		yy_pv = yypv;
		yy_ps = yyps;
		yy_state = yystate;

		/*
		** top of for (;;) loop while no reductions done
		*/
	yy_stack:
		/*
		** put a state and value onto the stacks
		*/
#if YYDEBUG
		/*
		** if debugging, look up token value in list of value vs.
		** name pairs.  0 and negative (-1) are special values.
		** Note: linear search is used since time is not a real
		** consideration while debugging.
		*/
		if ( yydebug )
		{
			register int yy_i;

			printf( "State %d, token ", yy_state );
			if ( yychar == 0 )
				printf( "end-of-file\n" );
			else if ( yychar < 0 )
				printf( "-none-\n" );
			else
			{
				for ( yy_i = 0; yytoks[yy_i].t_val >= 0;
					yy_i++ )
				{
					if ( yytoks[yy_i].t_val == yychar )
						break;
				}
				printf( "%s\n", yytoks[yy_i].t_name );
			}
		}
#endif /* YYDEBUG */
		if ( ++yy_ps >= &yys[ yymaxdepth ] )	/* room on stack? */
		{
			int yynewmax, yys_off;

			/* The following pointer-differences are safe, since
			 * yypvt, yy_pv, and yypv all are a multiple of
			 * sizeof(YYSTYPE) bytes from yyv.
			 */
			int yypvt_off = yypvt - yyv;
			int yy_pv_off = yy_pv - yyv;
			int yypv_off = yypv - yyv;

			int *yys_base = yys;
#ifdef YYEXPAND
			yynewmax = YYEXPAND(yymaxdepth);
#else
			yynewmax = 2 * yymaxdepth;	/* double table size */
			if (yymaxdepth == YYMAXDEPTH)	/* first time growth */
			{
				void *newyys = YYNEW(int);
				void *newyyv = YYNEW(YYSTYPE);
				if (newyys != 0 && newyyv != 0)
				{
					yys = YYCOPY(newyys, yys, int);
					yyv = YYCOPY(newyyv, yyv, YYSTYPE);
				}
				else
					yynewmax = 0;	/* failed */
			}
			else				/* not first time */
			{
				yys = YYENLARGE(yys, int);
				yyv = YYENLARGE(yyv, YYSTYPE);
				if (yys == 0 || yyv == 0)
					yynewmax = 0;	/* failed */
			}
#endif
			if (yynewmax <= yymaxdepth)	/* tables not expanded */
			{
#ifdef __cplusplus
				yyerror( "yacc stack overflow" );
#else
				yyerror( "Yacc stack overflow" );
#endif
				YYABORT;
			}
			yymaxdepth = yynewmax;

			/* reset pointers into yys */
			yys_off = yys - yys_base;
			yy_ps = yy_ps + yys_off;
			yyps = yyps + yys_off;

			/* reset pointers into yyv */
			yypvt = yyv + yypvt_off;
			yy_pv = yyv + yy_pv_off;
			yypv = yyv + yypv_off;
		}
		*yy_ps = yy_state;
		*++yy_pv = yyval;

		/*
		** we have a new state - find out what to do
		*/
	yy_newstate:
		if ( ( yy_n = yypact[ yy_state ] ) <= YYFLAG )
			goto yydefault;		/* simple state */
#if YYDEBUG
		/*
		** if debugging, need to mark whether new token grabbed
		*/
		yytmp = yychar < 0;
#endif
		if ( ( yychar < 0 ) && ( ( yychar = yylex() ) < 0 ) )
			yychar = 0;		/* reached EOF */
#if YYDEBUG
		if ( yydebug && yytmp )
		{
			register int yy_i;

			printf( "Received token " );
			if ( yychar == 0 )
				printf( "end-of-file\n" );
			else if ( yychar < 0 )
				printf( "-none-\n" );
			else
			{
				for ( yy_i = 0; yytoks[yy_i].t_val >= 0;
					yy_i++ )
				{
					if ( yytoks[yy_i].t_val == yychar )
						break;
				}
				printf( "%s\n", yytoks[yy_i].t_name );
			}
		}
#endif /* YYDEBUG */
		if ( ( ( yy_n += yychar ) < 0 ) || ( yy_n >= YYLAST ) )
			goto yydefault;
		if ( yychk[ yy_n = yyact[ yy_n ] ] == yychar )	/*valid shift*/
		{
			yychar = -1;
			yyval = yylval;
			yy_state = yy_n;
			if ( yyerrflag > 0 )
				yyerrflag--;
			goto yy_stack;
		}

	yydefault:
		if ( ( yy_n = yydef[ yy_state ] ) == -2 )
		{
#if YYDEBUG
			yytmp = yychar < 0;
#endif
			if ( ( yychar < 0 ) && ( ( yychar = yylex() ) < 0 ) )
				yychar = 0;		/* reached EOF */
#if YYDEBUG
			if ( yydebug && yytmp )
			{
				register int yy_i;

				printf( "Received token " );
				if ( yychar == 0 )
					printf( "end-of-file\n" );
				else if ( yychar < 0 )
					printf( "-none-\n" );
				else
				{
					for ( yy_i = 0;
						yytoks[yy_i].t_val >= 0;
						yy_i++ )
					{
						if ( yytoks[yy_i].t_val
							== yychar )
						{
							break;
						}
					}
					printf( "%s\n", yytoks[yy_i].t_name );
				}
			}
#endif /* YYDEBUG */
			/*
			** look through exception table
			*/
			{
				register int *yyxi = yyexca;

				while ( ( *yyxi != -1 ) ||
					( yyxi[1] != yy_state ) )
				{
					yyxi += 2;
				}
				while ( ( *(yyxi += 2) >= 0 ) &&
					( *yyxi != yychar ) )
					;
				if ( ( yy_n = yyxi[1] ) < 0 )
					YYACCEPT;
			}
		}

		/*
		** check for syntax error
		*/
		if ( yy_n == 0 )	/* have an error */
		{
			/* no worry about speed here! */
			switch ( yyerrflag )
			{
			case 0:		/* new error */
#ifdef __cplusplus
				yyerror( "syntax error" );
#else
				yyerror( "Syntax error" );
#endif
				goto skip_init;
			yyerrlab:
				/*
				** get globals into registers.
				** we have a user generated syntax type error
				*/
				yy_pv = yypv;
				yy_ps = yyps;
				yy_state = yystate;
				yynerrs++;
				/* FALLTHRU */
			skip_init:
			case 1:
			case 2:		/* incompletely recovered error */
					/* try again... */
				yyerrflag = 3;
				/*
				** find state where "error" is a legal
				** shift action
				*/
				while ( yy_ps >= yys )
				{
					yy_n = yypact[ *yy_ps ] + YYERRCODE;
					if ( yy_n >= 0 && yy_n < YYLAST &&
						yychk[yyact[yy_n]] == YYERRCODE)					{
						/*
						** simulate shift of "error"
						*/
						yy_state = yyact[ yy_n ];
						goto yy_stack;
					}
					/*
					** current state has no shift on
					** "error", pop stack
					*/
#if YYDEBUG
#	define _POP_ "Error recovery pops state %d, uncovers state %d\n"
					if ( yydebug )
						printf( _POP_, *yy_ps,
							yy_ps[-1] );
#	undef _POP_
#endif
					yy_ps--;
					yy_pv--;
				}
				/*
				** there is no state on stack with "error" as
				** a valid shift.  give up.
				*/
				YYABORT;
			case 3:		/* no shift yet; eat a token */
#if YYDEBUG
				/*
				** if debugging, look up token in list of
				** pairs.  0 and negative shouldn't occur,
				** but since timing doesn't matter when
				** debugging, it doesn't hurt to leave the
				** tests here.
				*/
				if ( yydebug )
				{
					register int yy_i;

					printf( "Error recovery discards " );
					if ( yychar == 0 )
						printf( "token end-of-file\n" );
					else if ( yychar < 0 )
						printf( "token -none-\n" );
					else
					{
						for ( yy_i = 0;
							yytoks[yy_i].t_val >= 0;
							yy_i++ )
						{
							if ( yytoks[yy_i].t_val
								== yychar )
							{
								break;
							}
						}
						printf( "token %s\n",
							yytoks[yy_i].t_name );
					}
				}
#endif /* YYDEBUG */
				if ( yychar == 0 )	/* reached EOF. quit */
					YYABORT;
				yychar = -1;
				goto yy_newstate;
			}
		}/* end if ( yy_n == 0 ) */
		/*
		** reduction by production yy_n
		** put stack tops, etc. so things right after switch
		*/
#if YYDEBUG
		/*
		** if debugging, print the string that is the user's
		** specification of the reduction which is just about
		** to be done.
		*/
		if ( yydebug )
			printf( "Reduce by (%d) \"%s\"\n",
				yy_n, yyreds[ yy_n ] );
#endif
		yytmp = yy_n;			/* value to switch over */
		yypvt = yy_pv;			/* $vars top of value stack */
		/*
		** Look in goto table for next state
		** Sorry about using yy_state here as temporary
		** register variable, but why not, if it works...
		** If yyr2[ yy_n ] doesn't have the low order bit
		** set, then there is no action to be done for
		** this reduction.  So, no saving & unsaving of
		** registers done.  The only difference between the
		** code just after the if and the body of the if is
		** the goto yy_stack in the body.  This way the test
		** can be made before the choice of what to do is needed.
		*/
		{
			/* length of production doubled with extra bit */
			register int yy_len = yyr2[ yy_n ];

			if ( !( yy_len & 01 ) )
			{
				yy_len >>= 1;
				yyval = ( yy_pv -= yy_len )[1];	/* $$ = $1 */
				yy_state = yypgo[ yy_n = yyr1[ yy_n ] ] +
					*( yy_ps -= yy_len ) + 1;
				if ( yy_state >= YYLAST ||
					yychk[ yy_state =
					yyact[ yy_state ] ] != -yy_n )
				{
					yy_state = yyact[ yypgo[ yy_n ] ];
				}
				goto yy_stack;
			}
			yy_len >>= 1;
			yyval = ( yy_pv -= yy_len )[1];	/* $$ = $1 */
			yy_state = yypgo[ yy_n = yyr1[ yy_n ] ] +
				*( yy_ps -= yy_len ) + 1;
			if ( yy_state >= YYLAST ||
				yychk[ yy_state = yyact[ yy_state ] ] != -yy_n )
			{
				yy_state = yyact[ yypgo[ yy_n ] ];
			}
		}
					/* save until reenter driver code */
		yystate = yy_state;
		yyps = yy_ps;
		yypv = yy_pv;
	}
	/*
	** code supplied by user is placed in this switch
	*/
	switch( yytmp )
	{
		
case 1:
# line 31 "parse.y"
{ ADD_OP(OP_STOP); return 0; } break;
case 2:
# line 32 "parse.y"
{ return 1; } break;
case 5:
# line 36 "parse.y"
{ ADD_OP(OP_POP); } break;
case 6:
# line 38 "parse.y"
{ ADD_OP(OP_PUSH_VAR); ADD_SYM(yypvt[-2]);
			  ADD_OP(OP_ASSIGN); yypvt[-2]->type = DEFINED; } break;
case 7:
# line 41 "parse.y"
{ ADD_OP(OP_PUSH_CONST); ADD_SYM(yypvt[-0]); } break;
case 8:
# line 42 "parse.y"
{ ADD_OP(OP_PUSH_VAR); ADD_SYM(yypvt[-0]); ADD_OP(OP_EVAL); } break;
case 10:
# line 44 "parse.y"
{ ADD_OP(OP_BUILTIN); ADD_SYM(yypvt[-3]); } break;
case 12:
# line 46 "parse.y"
{ ADD_OP(OP_ADD); } break;
case 13:
# line 47 "parse.y"
{ ADD_OP(OP_SUB); } break;
case 14:
# line 48 "parse.y"
{ ADD_OP(OP_MUL); } break;
case 15:
# line 49 "parse.y"
{ ADD_OP(OP_DIV); } break;
case 16:
# line 50 "parse.y"
{ ADD_OP(OP_POWER); } break;
case 17:
# line 51 "parse.y"
{ ADD_OP(OP_NEGATE); } break;
case 18:
# line 52 "parse.y"
{ ADD_OP(OP_GT); } break;
case 19:
# line 53 "parse.y"
{ ADD_OP(OP_GE); } break;
case 20:
# line 54 "parse.y"
{ ADD_OP(OP_LT); } break;
case 21:
# line 55 "parse.y"
{ ADD_OP(OP_LE); } break;
case 22:
# line 56 "parse.y"
{ ADD_OP(OP_EQ); } break;
case 23:
# line 57 "parse.y"
{ ADD_OP(OP_NE); } break;
case 24:
# line 58 "parse.y"
{ ADD_OP(OP_AND); } break;
case 25:
# line 59 "parse.y"
{ ADD_OP(OP_OR); } break;
case 26:
# line 60 "parse.y"
{ ADD_OP(OP_NOT); } break;
	}
	goto yystack;		/* reset registers in driver code */
}
