#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "histoscope.h"
float rndom(void);
/*
 *  ContinuousDemo.c - program to generate random n-tuple and histogram data
 */
#define TRUE 1
#define NUM_PTS 10000.
main()
{
    float values[8], r1, r2, r3, num_pts = NUM_PTS;
    double d;
    int   n_variables = 8, cps = 10;
    int   i, j, k, ntuple_id, hs1_id, hs2_id, indic_id, ctrl_id, trigr_id;
    static char *names[8] = { "X", "Y", "index", "sqrt(|X|)", "cos(Y)",
    		              "log(|X|)", "sqrt(|X|)-log(|Y|)", "sin(index)" };

    hs_initialize("ContinuousDemo.c");
    ntuple_id = hs_create_ntuple(101, "Example Ntuple", "HS", n_variables,
    		names);
    hs1_id    = hs_create_1d_hist(201, "Example 1d Hist", "HS", "Y", "total",
		100, -3.1, 3.1);
    hs2_id    = hs_create_2d_hist(301, "2d Hist Example", "HS","X", "Y", 
		"total", 25, 25, -3.1, 3.1, -3.1, 3.1);
    indic_id  = hs_create_indicator(401, "Example Indicator", "HS", 0., 10000.);
    ctrl_id   = hs_create_control(501, "Control: Num of points", "HS", 0.,
    		NUM_PTS, NUM_PTS);
    trigr_id  = hs_create_trigger(601, "Trigger Example", "HS");

    while (1) {
      
      for (i = 1; i <= num_pts; ++i) {
    
    	/* Random data computation - This could be any code you wish */
	while (1) {
	    r1 = rndom();
    	    r2 = rndom();
    	    r1 = 2.0 * r1 - 1.0;
    	    r2 = 2.0 * r2 - 1.0;
    	    r3 = r1 * r1 + r2 * r2;
    	    if (r3 <= 1.0)
    	        break;
	}
	values[0] = r1 * sqrt((-2.0*log(r3))/r3);
    	values[1] = r2 * sqrt((-2.0*log(r3))/r3);
    	values[2] = i;
    	values[3] = sqrt(fabs(values[0]));
    	values[4] = cos(values[1]);
    	values[5] = log(fabs(values[0]));
    	values[6] = sqrt(fabs(values[0])) - log(fabs(values[1]));
    	values[7] = sin(values[2]);

    	hs_fill_ntuple(ntuple_id, values);
    	hs_fill_1d_hist(hs1_id, values[1], 1.);
    	hs_fill_2d_hist(hs2_id, values[0], values[1], 1.);
    	hs_set_indicator(indic_id, (float) i);
    	hs_update();

    	/*  Sleep 1 second per ten fills (or # fills if < 10) */
    	if (modf( ((float)i) / (float)cps, &d) == 0.) {
    	    sleep(1); 
    	    hs_read_control(ctrl_id, &num_pts);
    	    if (num_pts < cps)
    	    	cps = num_pts - 1;
    	    if (num_pts >= 10)
    	    	cps = 10;
    	    if (hs_check_trigger(trigr_id) != 0)
                break;
	}
	
      }
      hs_reset(ntuple_id);
    }
}

float rndom(void) 
{
   static int a = 16807, m = 2147483647, q = 127773, r = 2836, iseed = 19283755;

      int hi, lo, test;
      double d_lo;

      hi = (int)((float)iseed / (float)q);
      modf((double)iseed / (double)q, &d_lo);
      lo = iseed - ((int) d_lo) * q;
      test = a * lo - r * hi;
      if(test > 0) 
        iseed = test;
      else
        iseed = test + m;
      return ((float)(iseed))/((float)m);
}	
