/*******************************************************************************
*									       *
* clientXDRsocket.c -- Histo-Scope module called by HistoClient.c for all XDR  *
*		 and socket operations.  This separate module is put into its  *
*		 own shareable image to hide the Multinet bcopy routine from   *
*		 the client program.  Thus even users of YBOS can use Histo-   *
*		 Scope.							       *
*									       *
* Copyright (c) 1993 Universities Research Association, Inc.		       *
* All rights reserved.							       *
* 									       *
* This material resulted from work developed under a Government Contract and   *
* is subject to the following license:  The Government retains a paid-up,      *
* nonexclusive, irrevocable worldwide license to reproduce, prepare derivative *
* works, perform publicly and display publicly by or for the Government,       *
* including the right to distribute to other Government contractors.  Neither  *
* the United States nor the United States Department of Energy, nor any of     *
* their employees, makes any warranty, express or implied, or assumes any      *
* legal liability or responsibility for the accuracy, completeness, or         *
* usefulness of any information, apparatus, product, or process disclosed, or  *
* represents that its use would not infringe privately owned rights.           *
*                                        				       *
* Fermilab Nirvana GUI Library						       *
* April 27, 1993							       *
*									       *
* Written by Joy Kyriakopulos						       *
*									       *
* Modification History:							       *
*									       *
*									       *
*******************************************************************************/

#ifdef VMS

#include <stdio.h>
#include <stdlib.h>
#include <unixio.h>
#include <file.h>
#include <perror.h>
#include <mulsys/types.h>
#include <mulsys/socket.h>
#include <mul/netdb.h>
#include <mulsys/ioctl.h>
#include <mul/errno.h>
#include <netinet/in.h>
#include <string.h>
#define DONT_DECLARE_MALLOC
#include <rpc/types.h>
#include <errno.h>
#include <rpc/xdr.h>
#define XDR_FILTERS_HERE 1
#include "../histo_util/histo.h"
#include "../histo_util/xdrHisto.h"
#include "../histo_util/histprotocol.h"

bool_t hs_xdr_string(XDR *xdrs, char **string, int size)
{
    return (xdr_string(xdrs, string, size));
}

bool_t hs_xdr_wrapstring(XDR *xdrs, char **string)
{
    return (xdr_wrapstring(xdrs, string));
}

void   hs_xdrmem_create(XDR *xdrs, char *buf, int size, enum xdr_op op)
{
    xdrmem_create(xdrs, buf, size, op);
}

void   hs_xdrstdio_create(XDR *xdrs, char *file, enum xdr_op op)
{
    xdrstdio_create(xdrs, file, op);
}

void   hs_xdr_destroy(XDR *xdrs)
{
    xdr_destroy(xdrs);
}

bool_t   hs_xdr_int(XDR *xdrs, int *ival)
{
    return xdr_int(xdrs, ival);
}

bool_t   hs_xdr_float(XDR *xdrs, int *fval)
{
    return xdr_float(xdrs, fval);
}

bool_t   hs_xdr_array(XDR *xdrs, char **arrp, int sizep, int maxsize, 
		int elsize, xdrproc_t elproc)
{
    return xdr_array(xdrs, arrp, sizep, maxsize, elsize, elproc);
}

int    hs_socket_read(unsigned short fd, char *buf, unsigned int bytesToRead)
{
    return socket_read(fd, buf, bytesToRead);
}

int hs_socket_write(unsigned short fd, char *buf, unsigned int bytesToWrite)
{
    return socket_write(fd, buf, bytesToWrite);
}

int hs_socket_close(unsigned short VMS_Channel)
{
    return socket_close(VMS_Channel);
}

void   hs_socket_perror(char *msg)
{
    socket_perror(msg);
}

int    hs_socket_errno(void)
{
    return socket_errno;
}

int hs_socket_ioctl(unsigned short fd, unsigned int request, int *enableFlag)
{
    return socket_ioctl(fd, request, enableFlag);
}

int    hs_getsockopt(unsigned short fd, unsigned int level, unsigned int
				option, char *optVal, unsigned int optLen)
{
    return getsockopt(fd, level, option, optVal, optLen);
}

int    hs_setsockopt(unsigned short fd, unsigned int level, unsigned int
				option, char *optVal, unsigned int optLen)
{
    return setsockopt(fd, level, option, optVal, optLen);
}

int hs_accept(unsigned short fd, struct sockaddr *addr, unsigned int *addrLen)
{
    return accept(fd, addr, addrLen);
}

int hs_socket(unsigned int addrFam, unsigned int type, unsigned int protocol)
{
    return socket(addrFam, type, protocol);
}

int hs_bind(unsigned short fd, struct sockaddr *name, unsigned int nameLen)
{
    return bind(fd, name, nameLen);
}

int hs_getsockname(unsigned short fd, struct sockaddr *addr, 
			unsigned int *addrLen)
{
    return getsockname(fd, addr, addrLen);
}

int hs_listen(unsigned short fd, unsigned int backlog)
{
    return listen(fd, backlog);
}

extern char *hs_vms_errno_string(void)
{
    return vms_errno_string();
}

unsigned long hs_ntohs(unsigned long val)
{
    return ntohs(val);
}

extern char * hs_gethostbyname(char *name)
{
    struct hostent *hp;
    
    hp = gethostbyname(name);
    return hp->h_name;
}

extern int hs_gethostname(char *name, unsigned int length)
{
    return gethostname(name, length);
}

#endif /*VMS*/
