/*******************************************************************************
*									       *
* fortAPIBinding.c - FORTRAN interface for Histo-Scope client routines, API    *
*									       *
*									       *
*		     This file contains the following functions:	       *
*									       *
*	 hs_id               - Return the hs id from uid and Category	       *
*	 hs_id_from_title    - Return the hs id from title & category.         *
*	 hs_list_items       - Return the list of items from title & Category  *
*			       number of variables and automatic storage       *
* Copyright (c) 1993, 1994 Universities Research Association, Inc.	       *
* All rights reserved.							       *
* 									       *
* This material resulted from work developed under a Government Contract and   *
* is subject to the following license:  The Government retains a paid-up,      *
* nonexclusive, irrevocable worldwide license to reproduce, prepare derivative *
* works, perform publicly and display publicly by or for the Government,       *
* including the right to distribute to other Government contractors.  Neither  *
* the United States nor the United States Department of Energy, nor any of     *
* their employees, makes any warranty, express or implied, or assumes any      *
* legal liability or responsibility for the accuracy, completeness, or         *
* usefulness of any information, apparatus, product, or process disclosed, or  *
* represents that its use would not infringe privately owned rights.           *
*                                        				       *
* Fermilab Nirvana GUI Library						       *
* October 30, 1992							       *
*									       *
*	P. Lebrun, December 13 1993 					       *
*									       *
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#ifdef VMS
#include <sys/descrip.h>
#endif /*VMS*/
#include "../histo_util/histo.h"
#include "../histo_util/histoUtil.h"
#include "HistoClient.h"
#include "HbookClient.h"
#include "histoApiArytH.h"
#include "histoApiFiles.h"
#include "histoApiHists.h"
#include "histoApiItems.h"
#include "histoApiNTs.h"
#include "histoApiStatH.h"

char *mallocNCopy(char *string, int length);
extern void cleanFortranString(char *string, int length);

#ifdef VMS
extern char **strDescArToNul(struct dsc$descriptor_a *vmsStrAr, int maxDim);
extern void FreeArPtrNul(char **strPtrAr);

int hs_id(int *uid, struct dsc$descriptor_s *vmsCatString)
{
    int length = vmsCatString->dsc$w_length;
    char *id_category = vmsCatString->dsc$a_pointer;
#else
int hs_id_(int *uid, char *id_category, int length)
{
#endif /*VMS*/
    char *catString;
    int idtmp;
    
    catString = mallocNCopy(id_category, length);
    idtmp = histo_id(*uid, catString);
    if (idtmp == -1)
    	fprintf(stderr, "hs_id:  Uid %d not found within category %s\n", *uid,
    		catString);  
    free(catString);
    return idtmp;
}

#ifdef VMS
int hs_id_from_title(struct dsc$descriptor_s *vmsTitString,
          struct dsc$descriptor_s *vmsCatString)
{
    int lengthc = vmsCatString->dsc$w_length;
    char *id_category = vmsCatString->dsc$a_pointer;
    int lengtht = vmsTitString->dsc$w_length;
    char *id_title = vmsTitString->dsc$a_pointer;
#else
int hs_id_from_title_(char *id_title, char *id_category,
                                     int lengtht, int lengthc)
{
#endif /*VMS*/
    char *catString, *titString;
    int idtmp;
    
    catString = mallocNCopy(id_category, lengthc);
    titString = mallocNCopy(id_title, lengtht);
    idtmp = histo_id_from_title(titString, catString);
    free(catString);
    free(titString);
    return idtmp;
}

#ifdef VMS
int hs_list_items(struct dsc$descriptor_s *vmsTitString,
          struct dsc$descriptor_s *vmsCatString, int *list, int *num,
          		   int *matchFlg)
{
    int lengthc = vmsCatString->dsc$w_length;
    char *id_category = vmsCatString->dsc$a_pointer;
    int lengtht = vmsTitString->dsc$w_length;
    char *id_title = vmsTitString->dsc$a_pointer;
#else
int hs_list_items_(char *id_title, char *id_category, int *list, int *num,
                           int *matchFlg, int lengtht, int lengthc)
{
#endif /*VMS*/
    char *CatString, *TitString;
    int idtmp;
    
    CatString = mallocNCopy(id_category, lengthc);
    TitString = mallocNCopy(id_title, lengtht);
    idtmp = histo_list_items(TitString, CatString, list, *num, *matchFlg);
    free(CatString);
    free(TitString);
    return idtmp;
}

#ifdef VMS
int hs_uid(int *id)
#else
int hs_uid_(int *id)
#endif /*VMS*/
{

   return histo_uid(*id);
}

#ifdef VMS
int hs_category(int *id, struct dsc$descriptor_s *vmsCatString)
{
    int lengthc = vmsCatString->dsc$w_length;
    char *id_category = vmsCatString->dsc$a_pointer;
#else
int hs_category_(int *id, char *id_category, int lengthc) 
{
#endif /*VMS*/
    char *CatString;
    int idtmp;
    
    CatString = malloc(HS_MAX_CATEGORY_LENGTH+1);
    idtmp = histo_category(*id, CatString);
    if (idtmp > 0) {
    	if (idtmp >= lengthc)
    	    /* just copy category up to length of FORTRAN string */
    	    strncpy(id_category, CatString, lengthc);
    	else {
    	    /* FORTRAN string is longer than category, pad with spaces */
    	    strcpy(id_category, CatString);
    	    memset(id_category+idtmp, ' ', lengthc-idtmp);
    	}
    }
    free(CatString);
    return idtmp;
}

#ifdef VMS
int hs_title(int *id, struct dsc$descriptor_s *vmsTitString)
{
    int lengtht = vmsTitString->dsc$w_length;
    char *id_title = vmsTitString->dsc$a_pointer;
#else
int hs_title_(int *id, char *id_title, int lengtht) 
{
#endif /*VMS*/
    char *TitString;
    int idtmp;
    
    TitString = malloc(HS_MAX_TITLE_LENGTH+1);
    idtmp = histo_title(*id, TitString);
    if (idtmp > 0) {
    	if (idtmp >= lengtht)
    	    /* just copy title up to length of FORTRAN string */
    	    strncpy(id_title, TitString, lengtht);
    	else {
    	    /* FORTRAN string is longer than title, pad with spaces */
    	    strcpy(id_title, TitString);
    	    memset(id_title+idtmp, ' ', lengtht-idtmp);
    	}
    }
    free(TitString);
    return idtmp;
}

#ifdef VMS
int hs_type(int *id)
#else
int hs_type_(int *id)
#endif /*VMS*/
{
   return (int) (histo_type(*id));
}

#ifdef VMS
int hs_read_file(struct dsc$descriptor_s *vmsfilename,
                 struct dsc$descriptor_s *vmsprefix)
{
    int lengthf = vmsfilename->dsc$w_length;
    char *file = vmsfilename->dsc$a_pointer;
    int lengthp = vmsprefix->dsc$w_length;
    char *pref = vmsprefix->dsc$a_pointer;
#else
int hs_read_file_(char *file, char *pref, int lengthf, int lengthp)
{
#endif /*VMS*/
    char *FileString, *PrefString;
    int idtmp;
    
    FileString = mallocNCopy(file, lengthf);
    if (pref != NULL)
	PrefString = mallocNCopy(pref, lengthp);
    else
	PrefString = NULL;
    idtmp = histo_read_file(FileString, PrefString);
    free(FileString);
    free(PrefString);
    return idtmp;
}


#ifdef VMS
int hs_read_file_items(struct dsc$descriptor_s *vmsfilename,
                      struct dsc$descriptor_s *vmsprefix, 
                      struct dsc$descriptor_s *vmscategory,
                      int *uids, int *n_uids )
{
    int lengthf = vmsfilename->dsc$w_length;
    char *file = vmsfilename->dsc$a_pointer;
    int lengthp = vmsprefix->dsc$w_length;
    char *pref = vmsprefix->dsc$a_pointer;
    int lengthc = vmscategory->dsc$w_length;
    char *category = vmscategory->dsc$a_pointer;
#else
int hs_read_file_items_(char *file, char *pref, char *category, 
         int *uids, int *n_uids, int lengthf, int lengthp, int lengthc )
{
#endif /*VMS*/
    char *FileString, *PrefString, *CatString;
    int idtmp;
    
    FileString = mallocNCopy(file, lengthf);
    PrefString = mallocNCopy(pref, lengthp);
    CatString = mallocNCopy(category, lengthc);
    idtmp = histo_read_file_items(FileString, PrefString, CatString,
    					uids, *n_uids);
    free(CatString);
    free(FileString);
    free(PrefString);
    return idtmp;
}


#ifdef VMS
int hs_save_file_items(struct dsc$descriptor_s *vmsfilename,
                      struct dsc$descriptor_s *vmscategory,
                      int *uids, int *n_uids )
{
    int lengthf = vmsfilename->dsc$w_length;
    char *file = vmsfilename->dsc$a_pointer;
    int lengthc = vmscategory->dsc$w_length;
    char *category = vmscategory->dsc$a_pointer;
#else
int hs_save_file_items_(char *file, char *category, 
         int *uids, int *n_uids, int lengthf, int lengthc )
{
#endif /*VMS*/
    char *FileString, *CatString;
    int idtmp;
    
    FileString = mallocNCopy(file, lengthf);
    CatString = mallocNCopy(category, lengthc);
    idtmp = histo_save_file_items(FileString, CatString,
    					uids, *n_uids);
    free(FileString);
    free(CatString);
    return idtmp;
}

#ifdef VMS
void hs_delete_category(struct dsc$descriptor_s *vmsCatString)
{
    int lengthc = vmsCatString->dsc$w_length;
    char *id_category = vmsCatString->dsc$a_pointer;
#else
void hs_delete_category_(char *id_category, int lengthc)
{
#endif /*VMS*/
    char *CatString;
    
    CatString = mallocNCopy(id_category, lengthc);
    histo_delete_category(CatString);
    free(CatString);
    return;
}

#ifdef VMS
void hs_change_uid(int *id, int *newuid)
#else
void hs_change_uid_(int *id, int *newuid)
#endif /*VMS*/
{
    int idt, newuidt;
    
    idt = *id; newuidt = *newuid;
    histo_change_uid(idt, newuidt);
    return;
}



#ifdef VMS
void hs_change_category(int *id, struct dsc$descriptor_s *vmsCatString)
{
    int lengthc = vmsCatString->dsc$w_length;
    char *id_category = vmsCatString->dsc$a_pointer;
#else
void hs_change_category_(int *id, char *id_category, int lengthc)
{
#endif /*VMS*/
    char *CatString;
    int idt;
    
    idt = *id;
    CatString = mallocNCopy(id_category, lengthc);
    histo_change_category(idt, CatString);
    free(CatString);
    return;
}

#ifdef VMS
void hs_change_title(int *id, struct dsc$descriptor_s *vmsTitString)
{
    int lengtht = vmsTitString->dsc$w_length;
    char *id_title = vmsTitString->dsc$a_pointer;
#else
void hs_change_title_(int *id, char *id_title, int lengtht)
{
#endif /*VMS*/
    char *TitString;
    int idt;
    
    idt = *id;
    TitString = mallocNCopy(id_title, lengtht);
    histo_change_title(idt, TitString);
    free(TitString);
    return;
}

#ifdef VMS
int hs_num_items(void)
{
#else
int hs_num_items_(void)
{
#endif /*VMS*/

    return histo_num_items();
}

#ifdef VMS
void hs_1d_hist_block_fill(int *id, float *data, float *err, float *err_m)
{
#else
void hs_1d_hist_block_fill_(int *id, float *data, float *err, float *err_m)
{
#endif /*VMS*/

   int idt;
   idt = *id;
   histo_1d_hist_block_fill(idt, data, err, err_m);
   return;
}

#ifdef VMS
void hs_2d_hist_block_fill(int *id, float *data, float *err, float *err_m)
{
#else
void hs_2d_hist_block_fill_(int *id, float *data, float *err, float *err_m)
{
#endif /*VMS*/

   int idt;
   idt = *id;
   histo_2d_hist_block_fill(idt, data, err, err_m, 1);
   return;
}

#ifdef VMS
int hs_1d_hist_num_bins(int *id)
#else
int hs_1d_hist_num_bins_(int *id)
#endif /*VMS*/
{
   int idt;
   idt = *id;
   return histo_1d_hist_num_bins(idt);
}

#ifdef VMS
void hs_2d_hist_num_bins(int *id, int *num_x_bins, int *num_y_bins)
#else
void hs_2d_hist_num_bins_(int *id, int *num_x_bins, int *num_y_bins)
#endif /*VMS*/
{
   int idt;
   idt = *id;
   histo_2d_hist_num_bins(idt, num_x_bins, num_y_bins);
   return;
}

#ifdef VMS
void hs_1d_hist_range(int *id, float *min, float *max)
#else
void hs_1d_hist_range_(int *id, float *min, float *max)
#endif /*VMS*/
{  
   int idt;
   idt = *id;
   histo_1d_hist_range(idt, min, max);
   return;
}

#ifdef VMS
void hs_2d_hist_range(int *id, float *x_min, float *x_max, float *y_min,
	float *y_max)
#else
void hs_2d_hist_range_(int *id, float *x_min, float *x_max, float *y_min,
	float *y_max)
#endif /*VMS*/
{
   int idt;
   idt = *id;
    histo_2d_hist_range(idt, x_min, x_max, y_min, y_max);
}

#ifdef VMS
void hs_1d_hist_bin_contents(int *id, float *data)
#else
void hs_1d_hist_bin_contents_(int *id, float *data)
#endif /*VMS*/
{
   int idt;
   idt = *id;
   histo_1d_hist_bin_contents(idt, data);
   return;
}

#ifdef VMS
void hs_2d_hist_bin_contents(int *id, float *data)
#else
void hs_2d_hist_bin_contents_(int *id, float *data)
#endif /*VMS*/
{
   int idt;
   idt = *id;
   histo_2d_hist_bin_contents(idt, data, 1);
   return; 
}

#ifdef VMS
int hs_1d_hist_errors(int *id, float *err, float *err_m)
#else
int hs_1d_hist_errors_(int *id, float *err, float *err_m)
#endif /*VMS*/
{
   int idt;
   idt = *id;
   
   return  histo_1d_hist_errors(idt, err, err_m);
}

#ifdef VMS
int hs_2d_hist_errors(int *id, float *err, float *err_m)
#else
int hs_2d_hist_errors_(int *id, float *err, float *err_m)
#endif /*VMS*/
{
   int idt;
   idt = *id;
   
   return histo_2d_hist_errors(idt, err, err_m, 1);
}

#ifdef VMS
void hs_1d_hist_overflows(int *id, float *underflow, float *overflow)
#else
void hs_1d_hist_overflows_(int *id, float *underflow, float *overflow)
#endif /*VMS*/
{
   int idt;
   idt = *id;
     histo_1d_hist_overflows(idt, underflow, overflow);
}

#ifdef VMS
void hs_2d_hist_overflows(int *id, float overflows[3][3])
#else
void hs_2d_hist_overflows_(int *id, float overflows[3][3])
#endif /*VMS*/
{
   int idt;
   idt = *id;
     histo_2d_hist_overflows(idt, overflows, 1);
}

#ifdef VMS
void hs_1d_hist_set_overflows(int *id, float *underflow, float *overflow)
#else
void hs_1d_hist_set_overflows_(int *id, float *underflow, float *overflow)
#endif /*VMS*/
{
     histo_1d_hist_set_overflows(*id, *underflow, *overflow);
}

#ifdef VMS
void hs_2d_hist_set_overflows(int *id, float overflows[3][3])
#else
void hs_2d_hist_set_overflows_(int *id, float overflows[3][3])
#endif /*VMS*/
{
     histo_2d_hist_set_overflows(*id, overflows, 1);
}

#ifdef VMS
int hs_num_entries(int *id)
#else
int hs_num_entries_(int *id)
#endif /*VMS*/
{
   int idt;
   idt = *id;
    return histo_num_entries(idt);
}

#ifdef VMS
float hs_1d_hist_x_value(int *id, float *x)
#else
float hs_1d_hist_x_value_(int *id, float *x)
#endif /*VMS*/
{
   int idt;
   float xt;
   idt = *id;
   xt = *x;
   return histo_1d_hist_x_value(idt, xt);
      
}

#ifdef VMS
float hs_2d_hist_xy_value(int *id, float *x, float *y)
#else
float hs_2d_hist_xy_value_(int *id, float *x, float *y)
#endif /*VMS*/
{
   int idt;
   float xt, yt;
   idt = *id;
   xt = *x; 
   yt = *y;
   return histo_2d_hist_xy_value(idt, xt, yt);
}

#ifdef VMS
float hs_1d_hist_bin_value(int *id, int *bin_num)
#else
float hs_1d_hist_bin_value_(int *id, int *bin_num)
#endif /*VMS*/
{
   int idt, jb;
   idt = *id;
   jb = *bin_num;
   return histo_1d_hist_bin_value(idt, jb, 1);
}

#ifdef VMS
float hs_2d_hist_bin_value(int *id, int *x_bin_num, int *y_bin_num)
#else
float hs_2d_hist_bin_value_(int *id, int *x_bin_num, int *y_bin_num)
#endif /*VMS*/
{
   int idt, jb, kb;
   idt = *id;
   jb = *x_bin_num;
   kb = *y_bin_num;
   return histo_2d_hist_bin_value(idt, jb, kb, 1);
}
   
#ifdef VMS
void hs_1d_hist_minimum(int *id, float *x, int *bin_num, float *value)
#else
void hs_1d_hist_minimum_(int *id, float *x, int *bin_num, float *value)
#endif /*VMS*/
{
     int idt;
     idt = *id;
     histo_1d_hist_minmax(idt, 0, x, bin_num, value, 1);
     return;
}
#ifdef VMS
void hs_2d_hist_minimum(int *id, float *x, float *y, int *x_bin_num,
	int *y_bin_num, float *value)
#else
void hs_2d_hist_minimum_(int *id, float *x, float *y, int *x_bin_num,
	int *y_bin_num, float *value)
#endif /*VMS*/
{
     int idt;
     idt = *id;
     histo_2d_hist_minmax(idt, 0, x, y, x_bin_num, y_bin_num, value, 1);
     return;
}

#ifdef VMS
void hs_1d_hist_maximum(int *id, float *x, int *bin_num, float *value)
#else
void hs_1d_hist_maximum_(int *id, float *x, int *bin_num, float *value)
#endif /*VMS*/
{
     int idt;
     idt = *id;
     histo_1d_hist_minmax(idt, 1, x, bin_num, value, 1);
     return;
}
#ifdef VMS
void hs_2d_hist_maximum(int *id, float *x, float *y, int *x_bin_num,
	int *y_bin_num, float *value)
#else
void hs_2d_hist_maximum_(int *id, float *x, float *y, int *x_bin_num,
	int *y_bin_num, float *value)
#endif /*VMS*/
{
     int idt;
     idt = *id;
     histo_2d_hist_minmax(idt, 1, x, y, x_bin_num, y_bin_num, value, 1);
     return;
}

#ifdef VMS
void hs_1d_hist_stats(int *id, float *mean, float *std_dev)
#else
void hs_1d_hist_stats_(int *id, float *mean, float *std_dev)
#endif /*VMS*/
{
     int idt = *id;
     histo_1d_hist_stats(idt, mean, std_dev);
     return;
}

#ifdef VMS
void hs_2d_hist_stats(int *id, float *x_mean, float *y_mean, float *x_std_dev,
	float *y_std_dev)
#else
void hs_2d_hist_stats_(int *id, float *x_mean, float *y_mean, float *x_std_dev,
	float *y_std_dev)
#endif /*VMS*/
{
     int idt = *id;
     histo_2d_hist_stats(idt, x_mean, y_mean, x_std_dev, y_std_dev);
     return;
}

#ifdef VMS
float hs_hist_integral(int *id)
#else
float hs_hist_integral_(int *id)
#endif /*VMS*/
{
     int idt = *id;
     return histo_hist_integral(idt);
} 
   
#ifdef VMS
void hs_hist_set_gauss_errors(int *id)
#else
void hs_hist_set_gauss_errors_(int *id)
#endif /*VMS*/
{
     int idt = *id;
     histo_hist_set_gauss_errors(idt);
     return;
}

#ifdef VMS
int hs_sum_histograms(int *uid, struct dsc$descriptor_s *vmstitle,
                      struct dsc$descriptor_s *vmscategory,
                   int *id1, int *id2, float *const1, float *const2)
{
    int lengtht = vmstitle->dsc$w_length;
    char *title = vmstitle->dsc$a_pointer;
    int lengthc = vmscategory->dsc$w_length;
    char *category = vmscategory->dsc$a_pointer;
#else
int hs_sum_histograms_(int *uid, char *title, char *category,
                 int *id1, int *id2, float *const1, float *const2,
                 int lengtht, int lengthc)
{
#endif /*VMS*/

    char *TitleString, *CatString;
    int uidt = *uid;
    int id1t = *id1;
    int id2t = *id2;
    float const1t = *const1;
    float const2t = *const2;
    int idr;
    
    TitleString = mallocNCopy(title, lengtht);
    CatString = mallocNCopy(category, lengthc);
    idr = histo_sum_histograms(uidt, TitleString, CatString,
                         id1t, id2t, const1t, const2t);
    free(TitleString);
    free(CatString);
    return idr;
}

#ifdef VMS
int hs_multiply_histograms(int *uid, struct dsc$descriptor_s *vmstitle,
                      struct dsc$descriptor_s *vmscategory,
                         int *id1, int *id2, float *consta)
{
    int lengtht = vmstitle->dsc$w_length;
    char *title = vmstitle->dsc$a_pointer;
    int lengthc = vmscategory->dsc$w_length;
    char *category = vmscategory->dsc$a_pointer;
#else
int hs_multiply_histograms_(int *uid, char *title, char *category,
            int *id1, int *id2, float *consta, int lengtht, int lengthc)
{
#endif /*VMS*/

    char *TitleString, *CatString;
    int uidt = *uid;
    int id1t = *id1;
    int id2t = *id2;
    float constat = *consta;
    int idr;
    
    TitleString = mallocNCopy(title, lengtht);
    CatString = mallocNCopy(category, lengthc);
    idr = histo_multdiv_histograms(uidt, TitleString, CatString,
                        0, id1t, id2t, constat);
    free(TitleString);
    free(CatString);
    return idr;
}

#ifdef VMS
int hs_1d_hist_derivative(int *uid, struct dsc$descriptor_s *vmstitle,
                      struct dsc$descriptor_s *vmscategory,
                      int *id )
{
    int lengtht = vmstitle->dsc$w_length;
    char *title = vmstitle->dsc$a_pointer;
    int lengthc = vmscategory->dsc$w_length;
    char *category = vmscategory->dsc$a_pointer;
#else
int hs_1d_hist_derivative_(int *uid, char *title, char *category,
                 int *id, int lengtht, int lengthc)
{
#endif /*VMS*/

    char *TitleString, *CatString;
    int uidt = *uid;
    int idt = *id;
    int idr;
    
    TitleString = mallocNCopy(title, lengtht);
    CatString = mallocNCopy(category, lengthc);
    idr = histo_1d_hist_derivative(uidt, TitleString, CatString,
                         idt);
    free(TitleString);
    free(CatString);
    return idr;
}

#ifdef VMS
int hs_divide_histograms(int *uid, struct dsc$descriptor_s *vmstitle,
                      struct dsc$descriptor_s *vmscategory,
                         int *id1, int *id2, float *consta)
{
    int lengtht = vmstitle->dsc$w_length;
    char *title = vmstitle->dsc$a_pointer;
    int lengthc = vmscategory->dsc$w_length;
    char *category = vmscategory->dsc$a_pointer;
#else
int hs_divide_histograms_(int *uid, char *title, char *category,
            int *id1, int *id2, float *consta, int lengtht, int lengthc)
{
#endif /*VMS*/

    char *TitleString, *CatString;
    int uidt = *uid;
    int id1t = *id1;
    int id2t = *id2;
    float constat = *consta;
    int idr;
    
    TitleString = mallocNCopy(title, lengtht);
    CatString = mallocNCopy(category, lengthc);
    idr = histo_multdiv_histograms(uidt, TitleString, CatString,
                        1, id1t, id2t, constat);
    free(TitleString);
    free(CatString);
    return idr;
    
}

#ifdef VMS
int hs_num_variables(int *id)
#else
int hs_num_variables_(int *id)
#endif /*VMS*/
{
    int idt = *id;
    return histo_num_variables(idt);
}

#ifdef VMS
int hs_variable_name(int *id, int *column, struct dsc$descriptor_s *vmsname)
{
    int lengthn = vmsname->dsc$w_length;
    char *name = vmsname->dsc$a_pointer;
#else
int hs_variable_name_(int *id, int *column, char *name, int lengthn)
{
#endif /*VMS*/
     int idt = *id;
     int columnt = *column;
     int ret_val;
     char *NameString;
     
     NameString = (char *) malloc(HS_MAX_NAME_LENGTH+1);
     ret_val = histo_variable_name(idt, columnt, NameString, 1);
     strcpy(name, NameString);
     free(NameString);
     return ret_val;
}

#ifdef VMS
int hs_variable_index(int *id, struct dsc$descriptor_s *vmsname)
{
    int lengthn = vmsname->dsc$w_length;
    char *name = vmsname->dsc$a_pointer;
#else
int hs_variable_index_(int *id, char *name, int lengthn)
{
#endif /*VMS*/
     int idt = *id;
     int kk;
     char *NameString;
     
     NameString = mallocNCopy(name, lengthn);
     kk = histo_variable_index(idt, NameString, 1);
     free(NameString);
     return kk;
}

#ifdef VMS
float hs_ntuple_value(int *id, int *row, int *column)
#else
float hs_ntuple_value_(int *id, int *row, int *column)
#endif /*VMS*/
{
     int idt = *id;
     int columnt = *column;
     int rowt = *row;
     return histo_ntuple_value(idt, rowt, columnt, 1);
}

#ifdef VMS
void hs_ntuple_contents(int *id, float *data)
#else
void hs_ntuple_contents_(int *id, float *data)
#endif /*VMS*/
{
     int idt = *id;
     histo_ntuple_contents(idt, data, 1);
     return;
}

#ifdef VMS
void hs_row_contents(int *id, int *row, float *data)
#else
void hs_row_contents_(int *id, int *row, float *data)
#endif /*VMS*/
{
     int idt = *id;
     int rowt = *row;
     histo_row_contents(idt, rowt, data, 1);
     return;
}

#ifdef VMS
void hs_column_contents(int *id, int *column, float *data)
#else
void hs_column_contents_(int *id, int *column, float *data)
#endif /*VMS*/
{
     int idt = *id;
     int columnt = *column;
     histo_column_contents(idt, columnt, data, 1);
     return;
}

#ifdef VMS
int hs_merge_entries(int *uid, struct dsc$descriptor_s *vmstitle,
                      struct dsc$descriptor_s *vmscategory,
                      int *id1, int *id2)
{
    int lengtht = vmstitle->dsc$w_length;
    char *title = vmstitle->dsc$a_pointer;
    int lengthc = vmscategory->dsc$w_length;
    char *category = vmscategory->dsc$a_pointer;
#else
int hs_merge_entries_(int *uid, char *title, char *category,
                          int *id1, int *id2, int lengtht, int lengthc)

{
#endif /*VMS*/
    char *TitleString, *CatString;
    int uidt = *uid;
    int id1t = *id1;
    int id2t = *id2;
    int idr;
     
    TitleString = mallocNCopy(title, lengtht);
    CatString = mallocNCopy(category, lengthc);
    idr = histo_merge_entries(uidt, TitleString, CatString, id1t, id2t);
    free(TitleString);
    free(CatString);
    return idr;
}

#ifdef VMS
void hs_sum_category(struct dsc$descriptor_s *vmscat_top1, 
                     struct dsc$descriptor_s *vmscat_top2,
                     struct dsc$descriptor_s *vmsprefixsum)
{
    int lengtht1 = vmscat_top1->dsc$w_length;
    char *cat_top1 = vmscat_top1->dsc$a_pointer;
    int lengtht2 = vmscat_top2->dsc$w_length;
    char *cat_top2 = vmscat_top2->dsc$a_pointer;
    int lengthp = vmsprefixsum->dsc$w_length;
    char *prefixsum = vmsprefixsum->dsc$a_pointer;

#else
void hs_sum_category_(char *cat_top1, char *cat_top2,
            char *prefixsum, int lengtht1, int lengtht2, int lengthp)
{
#endif /*VMS*/
char *cat_top1string, *cat_top2string, *prefixsumstring; 

     cat_top1string = mallocNCopy(cat_top1, lengtht1);
     cat_top2string = mallocNCopy(cat_top2, lengtht2);
     prefixsumstring = mallocNCopy(prefixsum, lengthp);
     histo_sum_category(cat_top1string, cat_top2string, prefixsumstring);
     free(cat_top1string);
     free(cat_top2string);
     free(prefixsumstring);
     return;
}
 
#ifdef VMS
void hs_sum_file(struct dsc$descriptor_s *vmsfile, 
                     struct dsc$descriptor_s *vmscat_top,
                     struct dsc$descriptor_s *vmsprefixsum)
{
    int lengthf = vmsfile->dsc$w_length;
    char *file = vmsfile->dsc$a_pointer;
    int lengtht = vmscat_top->dsc$w_length;
    char *cat_top = vmscat_top->dsc$a_pointer;
    int lengthp = vmsprefixsum->dsc$w_length;
    char *prefixsum = vmsprefixsum->dsc$a_pointer;

#else
void hs_sum_file_(char *file, char *cat_top,
            char *prefixsum, int lengthf, int lengtht, int lengthp)
{
#endif /*VMS*/
char *filestring, *cat_topstring, *prefixsumstring; 

     filestring = mallocNCopy(file, lengthf);
     cat_topstring = mallocNCopy(cat_top, lengtht);
     prefixsumstring = mallocNCopy(prefixsum, lengthp);
     histo_sum_file(filestring, cat_topstring, prefixsumstring);
     free(filestring);
     free(cat_topstring);
     free(prefixsumstring);
     return;
}

