/*******************************************************************************
*									       *
* fortranBinding.c - FORTRAN Linux interface for Histo-Scope client API        *
*									       *
*		     This file is necessary because some Linux FORTRAN         *
*		     compilers put TWO underscores after routine names	       *
*									       *
* Copyright (c) 1997 Universities Research Association, Inc.		       *
* All rights reserved.							       *
* 									       *
* This material resulted from work developed under a Government Contract and   *
* is subject to the following license:  The Government retains a paid-up,      *
* nonexclusive, irrevocable worldwide license to reproduce, prepare derivative *
* works, perform publicly and display publicly by or for the Government,       *
* including the right to distribute to other Government contractors.  Neither  *
* the United States nor the United States Department of Energy, nor any of     *
* their employees, makes any warranty, express or implied, or assumes any      *
* legal liability or responsibility for the accuracy, completeness, or         *
* usefulness of any information, apparatus, product, or process disclosed, or  *
* represents that its use would not infringe privately owned rights.           *
*                                        				       *
* Fermilab Nirvana GUI Library						       *
* April 8, 1997								       *
*									       *
* Written by Joy Kyriakopulos						       *
*									       *
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include "fortBind.h"

void hs_initialize__(char *id_string, int length)
{
    hs_initialize_(id_string, length);
}

void hs_update__(void)
{
    hs_update_();
}

void hs_complete__(void)
{
    hs_complete_();
}

void hs_complete_and_wait__(void)
{
    hs_complete_and_wait_();
}

void hs_histoscope__(int *return_immediately)
{
    hs_histoscope_(return_immediately);
}

void hs_histo_with_config__(int *return_immediately, char *config_file, 
	int config_length)
{
    hs_histo_with_config_(return_immediately, config_file, config_length);
}

void hs_kill_histoscope__(void)
{
    hs_kill_histoscope_();
}

int hs_num_connected_scopes__(void)
{
    return hs_num_connected_scopes_();
}

void hs_load_config_string__(char *cfgStr, int cfgBufLen)
{
    hs_load_config_string_(cfgStr, cfgBufLen);
}

void hs_load_config_file__(char *cfgFile, int cfgFLen)
{
    hs_load_config_file_(cfgFile, cfgFLen);
}

int hs_create_1d_hist__(int *uid, char *title, char *category, char *x_label,
	   char *y_label, int *n_bins, float *min, float *max, int title_length,
	   int category_length, int x_label_length, int y_label_length)
{
    return hs_create_1d_hist_(uid, title, category, x_label, y_label, n_bins,
    				min, max, title_length, category_length, 
    				x_label_length, y_label_length);
}

int hs_create_2d_hist__(int *uid, char *title, char *category, 
			char *x_label,char *y_label,
			char *z_label, int *x_bins, int *y_bins, float *x_min,
			float *x_max, float *y_min, float *y_max, int title_length,
			int category_length, int x_label_length, int y_label_length,
			int z_label_length)
{
    return hs_create_2d_hist_(uid, title, category, x_label, y_label,
    			 z_label, x_bins, y_bins, x_min, x_max,
    			 y_min, y_max, title_length, category_length, 
    			 x_label_length, y_label_length, z_label_length);
}

int hs_create_ntuple__(int *uid, char *title, char *category, int *n_variables,
			char *ftnNames, int title_length, int category_length,
			int name_length)
{
    return hs_create_ntuple_(uid, title, category, n_variables, ftnNames,
			title_length, category_length, name_length);
}

int hs_create_indicator__(int *uid, char *title, char *category, 
			float *min, float *max, int title_length,
			int category_length)
{
    return hs_create_indicator_(uid, title, category, min, max, title_length,  
			category_length);
}

int hs_create_control__(int *uid, char *title, char *category, float *min,
			float *max, float *default_value, int title_length, 
			int category_length)
{
    return hs_create_control_(uid, title, category, min, max, default_value, 
			title_length, category_length);
}

int hs_create_trigger__(int *uid, char *title, char *category,
			int title_length, int category_length)
{
    return hs_create_trigger_(uid, title, category, title_length, 
			category_length);
}

int hs_create_group__(int *uid, char *title, char *category, int *groupType,
			int *numItems, int *itemId, int *errsDisp,
			int title_length, int category_length)
{
    return hs_create_group_(uid, title, category, groupType, numItems,
    			itemId, errsDisp, title_length, category_length);
}

void hs_fill_1d_hist__(int *id, float *x, float *weight)
{
    hs_fill_1d_hist_(id, x, weight);
}

void hs_fill_2d_hist__(int *id, float *x, float *y, float *weight)
{
    hs_fill_2d_hist_(id, x, y, weight);
}

int hs_fill_ntuple__(int *id, float *values)
{
    return (hs_fill_ntuple_(id, values));
}

void hs_read_control__(int *id, float *value)
{
    hs_read_control_(id, value);
}

int hs_check_trigger__(int *id)
{
    return (hs_check_trigger_(id));
}

void hs_set_indicator__(int *id, float *value)
{
    hs_set_indicator_(id, value);
}

void hs_set_1d_errors__(int *id, float *err_valsP, float *err_valsM)
{
    (hs_set_1d_errors_(id, err_valsP, err_valsM));
}

void hs_set_2d_errors__(int *id, float *err_valsP, float *err_valsM)
{
    /* Note:  just like HBOOK, we require the array to be dimensioned exactly
    	      as specified to hs_create_2d_hist: x_bins x y_bins. 	      */

    (hs_set_2d_errors_(id, err_valsP, err_valsM));
}

void hs_reset__(int *id)
{
   hs_reset_(id);
}

int hs_save_file__(char *name, int length)
{
    return hs_save_file_(name, length);
}

void hs_delete__(int *id)
{
    hs_delete_(id);
}

void hs_delete_items__(int *ids, int *num_ids)
{
    hs_delete_items_(ids, num_ids);
}

#ifdef LINKED_WITH_HBOOK
void hs_hbook_setup__(char *topDirectory, int length)
{
    hs_hbook_setup_(topDirectory, length);
}

void hs_reset_hbook_setup__(char *topDirectory, int length)
{
    hs_reset_hbook_setup_(topDirectory, length);

}

#endif    /* LINKED_WITH_HBOOK */

