/*******************************************************************************
*									       *
* tohbook.c -- This program converts a Histosope file to an RZ-Hbook file      *
* called "test1.hbook"                                                         *
* to run the program type:                                                     *
*                         rHstToHbook.exe  file.ext                            *
*                             						       *
*									       *
* 				                                               *
* Fermilab Nirvana GUI Library						       *
* April 27, 1995							       *
*									       *
* Written by Paul Lebrun						       *
*            Andy Beretvas                                                     *
*                                                                              *
* Modified by Igor Volobouev, 2002, to fix conversion of ntuples, eliminate    *
* memory leaks, to make correct HBOOK directory structure, etc.                *
*                                                                              *
*******************************************************************************/
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "histoscope.h"

void tohbf_(char *, int *, int);
#define HBKMAX 10000000         /* space in words for hbook        */
struct pawC {int h[HBKMAX];} pawc_;
 
int main(int argc, char *argv[])
{
    int  hlim = HBKMAX;
    int  LREC;                /* number of  words in a logical record        */
   
    int   i,j;
    int   k1,k2;              /* indices for inverting array of data         */
    int   k3;                 /* indices for category names                  */
    int   l;                  /* loop index for ntuple variable names        */
    int   l1;                 /* loop index for printing of variable names   */ 
    int   l2;                 /* index for a single ntuple variable          */
    int   l3;                 /* index over all events (num_entries)         */
    int   m0,m1,m2,m3;
    int   str_num;            /* number of strrings that result for parsing  */
    /* category            */              
   
    int  num_items;           /* number of items in Histo-scope file          */
    int  num_match;           /* number of items that match                   */
    int  *ids;                /* list of histoscope identification numbers    */
    int  matchFlg;            /* flag used for list of items                  */
    int  length;              /* number of characters in category             */
    int  length1;        /* number of characters in title                     */
    int  length_n;        /* number of characters in ntuple variable names    */
    int  id_user;             /* user specified identification number         */
    int  nwords;              /* nwords in an ntuple                          */
    int  *variable_length; 
    int  data_type;     /* 0= 1D histo, 1= 2D histo, 2= ntuple, 3= indicator,
			   4= control,  5= trigger,  6=none                   */
                            
    int  num_bins;        /* number of bins for 1 d histo                     */
    int  nh1;             /* number of 1 d histograms produced                */
    int  num_x_bins;      /* number of x bins in a 2 d histo                  */
    int  num_y_bins;      /* number of y bins in a 2 d histo                  */
    int  nh2;             /* number of 2 d histograms produced                */
    int  num_var;         /* number of variables in an ntuple                 */
    int  nnt;             /* number of ntuples produced                       */
    int  num_entries;     /* number of rows in this ntuple                    */
    int  status;         /* 0= No errors, 1= only pos errors, 2= both pos and
			    and neg errors, 3= not a 1 d histo                   */
    int   uid_supplied = 0;  /* starting uid # for items wo/UIDs	             */
                        
    float con_0 = 0.;
    float *ptd_data;               /* pointer to data for 1 d histo         */
    float *ptd_pos_errors;         /* pointer to positive errors 1 d histo  */
    float *ptd_neg_errors;         /* pointer to negative errors 1 d histo  */
    float min,max, min_u, max_o;   /* min and max for 1 d histogram         */
    float underflow, overflow;     /*             for 1 d histogram         */
    float *ptd_2d_data;            /* pointer to data for 2d histo          */
    float *ptd_2d_pos_errors;      /* pointer to positive errors 2 d histo  */
    float *ptd_2d_neg_errors;      /* pointer to negative errors 2 d histo  */
    float *ptd_2di_data;           /* pointer for fortran call (inverted)   */
    float *ptd_2d_errors;          /* pointer for fortran call (inverted)   */
    float x_min,x_max,y_min,y_max; /* min and max for 2 d histo             */
    float x_min_u,x_max_o,y_min_u,y_max_o; /* min and max for 2 d histo     */
    float x_mean,y_mean;           /* mean (x and y) for 2 d histo          */
    float x_std_dev,y_std_dev;     /* std deviation (x,y) for 2 d histo     */
    float overflows[3][3];         /* 3*3 matrix of overflows               */
    float *ptd_ntuple_row;        /* pointer to a row of ntuple data        */
       
    /* we add a character to the length of title_string and category          */
    /* for the termminating NULL                                              */
    char title_string[81];      /* Histo-scope title for an item            */
    char category[256];         /* Histo-scope category for an item         */
    char paw_0[262];
    char paw_1[262];
    char paw_2[262];
    char paw_3[262];              /* Hbook directory name                    */
    char old_paw0[262];           /* old hbook directory name                */
    char old_paw1[262];
    char old_paw2[262];
    char old_paw3[262];
    char *variable_name;         /* list of variable names in an ntuple      */
    char *variable_n ;           /* list contains names of length 8          */
    char * pt_name;               /* pointer used to construct file name      */
    char * file_hbook;          /* output file is the same as input but .rz */
                                /* input file mxyz.input---> mxyz.rz       */
   
    LREC = 1024;
    nwords = LREC;
    nh1 = nh2 = nnt =0;

#ifdef H_DEBUG   
    fprintf(stderr,"This is the start of the main program.\n");
#endif   /*   H_DEBUG   */     
      
/*     fp = fopen("output_rHstToHbook","w");                          */
/*     fprintf(fp,"This is the start of the main program.\n");        */
     
/** check input file name **/
    if(argc != 2)                                                        
    {
	fprintf(stderr," You did not specify an input file \n");
	fprintf(stderr," The input file should be a Histo-scope file\n");
	exit(EXIT_FAILURE);                                                            
    }
/**   input file name ok **/
    file_hbook = (char *) malloc(strlen(argv[1]) + strlen(".rz") + 1);
    /* Put hbook file in current directory, so strip off pathname... */
    /*   Then use extension of .rz instead of file's extension    */
    if ( (pt_name =  (char *) strrchr(argv[1], '/')) != NULL )
	strcpy(file_hbook, pt_name+1);
    else
	strcpy(file_hbook, argv[1]);
    if((pt_name =  (char *)strchr(file_hbook,'.')) != NULL)
	*pt_name = '\0';
    strcat(file_hbook,".rz");
#ifdef H_DEBUG   
    fprintf(stderr," the new file name is %s \n",file_hbook);
#endif   /*   H_DEBUG   */     
       
                                                                        
    hs_initialize("Converts_hs_to_hbook");
    hlimit_(&hlim);
      
    num_items = hs_read_file(argv[1],"");
    if (num_items < 0)
	/* hs_read_file will print an error message */
	exit(EXIT_FAILURE);
      
#ifdef H_DEBUG
    fprintf(stderr,"the number of items is %d \n",num_items);
#endif   /* H_DEBUG  */ 
             
    ids = (int *) malloc(num_items * sizeof(int));
      
     
    matchFlg = 1;     
    /* specify an inexact match */
                     
    num_match = hs_list_items("","...",ids,num_items,matchFlg);
#ifdef H_DEBUG
    fprintf(stderr,"The result of hs_list_items is %d \n",num_match);
#endif    /* H_DEBUG   */
    if (num_match <= 0) {
	fprintf(stderr,"No histograms or ntuples found in file %s\n",argv[1]);
	exit(EXIT_FAILURE);
    }
    old_paw0[0] = ' ';
    old_paw0[1] = 0;
    old_paw1[0] = ' ';
    old_paw1[1] = 0;
    old_paw2[0] = ' ';
    old_paw2[1] = 0;
    old_paw3[0] = ' ';
    old_paw3[1] = 0;
       
                   
  
    for (i=0; i<num_match; i++)       /* start of loop over items    */
    {
	hcdir_("//PAWC"," ",6,1);
	id_user = hs_uid(ids[i]);
	if (id_user <= 0) {
	    if (uid_supplied == 0)
		printf(
		    " Histograms or ntuples without uids will be assigned a number.\n");
	    id_user = ++uid_supplied;
	}
	length  = hs_category(ids[i],category);
	length1 = hs_title(ids[i],title_string);
	data_type = hs_type(ids[i]);
     
#ifdef H_DEBUG
	fprintf(stderr,"The user specified identification number is %d \n",
		id_user);
	fprintf(stderr," The user sepcified address is %p \n",&id_user);
	fprintf(stderr,"            \n");
	fprintf(stderr,"For item  %d the id is %d \n",i,ids[i]);
	fprintf(stderr,"The category name is %s \n",category);
	fprintf(stderr,"The length of the category name is %d \n",length);
	fprintf(stderr,"The title is: %s \n",title_string);
	fprintf(stderr,"The data type is: %d \n",data_type);
#endif   /*   H_DEBUG   */

	if( (data_type != HS_1D_HISTOGRAM) &&
	    (data_type != HS_2D_HISTOGRAM) &&
	    (data_type != HS_NTUPLE))  continue;
       
     
 	str_num = 0;
	m0 = m1 = m2 = m3 = 0;
	for(k3 = 0; k3 < length; k3++)
	{
	    if(category[k3] == '/')
	    {
		++str_num;
	    }
	    if(str_num > 3)
	    {
		fprintf(stderr," Category name is nested more than 4 levels \n");
		exit(EXIT_FAILURE);
	    } 
         
	    if(str_num == 0)
	    {
		paw_0[m0] = category[k3];
		++m0;
	    }
	    else if (str_num == 1 && category[k3] != '/')
	    {
		paw_1[m1] = category[k3];
		++m1; 
	    }
	    else if (str_num == 2 && category[k3] != '/')
	    {
		paw_2[m2] = category[k3];
		++m2;
	    }
	    else if (str_num == 3 && category[k3] != '/')
	    {
		paw_3[m3] = category[k3];
		++m3;
	    }  
                                           
	}   /* end of for loop on k3 */
	paw_0[m0] = 0;
	paw_1[m1] = 0;
	paw_2[m2] = 0;
	paw_3[m3] = 0;
      
      
#ifdef H_DEBUG         
	fprintf(stderr," HLDIR called\n");
	fprintf(stderr," the str_num is %d \n",str_num);
	fprintf(stderr," length of paw_0 %d %s \n",strlen(paw_0),paw_0);
	fprintf(stderr," length of paw_1 %d %s \n",strlen(paw_1),paw_1);
	fprintf(stderr," length of paw_2 %d %s \n",strlen(paw_2),paw_2);
#endif   /* H_Debug   */
/*      hldir_("//PAWC","T",6,1);    */
       
	if(str_num ==0)
	{         
	    if(strcmp(old_paw0,paw_0) != 0) {
		hmdir_(paw_0, "S", strlen(paw_0), 1);
		strcpy(old_paw0,paw_0);
      
	    }
	    else if (paw_0[0])
          hcdir_(paw_0," ",strlen(paw_0),1);  
	}
      
	if(str_num ==1)
	{                 /*   start of loop for str_num = 1 */
	    if(strcmp(old_paw0,paw_0) != 0)
	    {
		hmdir_(paw_0, "S", strlen(paw_0), 1);
		strcpy(old_paw0,paw_0);
	    }
	    else
		hcdir_(paw_0," ",strlen(paw_0),1);
      
	    if(strcmp(old_paw1,paw_1) !=0)
	    {
		hmdir_(paw_1,"S",strlen(paw_1),1);
		strcpy(old_paw1,paw_1);
	    }
	    else
		hcdir_(paw_1," ",strlen(paw_1),1);
         	                                
	}                 /*     end of loop for str_num =1 */
      
	if(str_num ==2)
	{                 /* start of loop for str_num = 2   */
	    if(strcmp(old_paw0,paw_0) != 0)
	    {
		hmdir_(paw_0, "S", strlen(paw_0), 1);
		strcpy(old_paw0,paw_0);
	    }
	    else
		hcdir_(paw_0," ",strlen(paw_0),1);
      
	    if(strcmp(old_paw1,paw_1) !=0)
	    {
		hmdir_(paw_1,"S",strlen(paw_1),1);
		strcpy(old_paw1,paw_1);
	    }
	    else
		hcdir_(paw_1," ",strlen(paw_1),1);
       
	    if(strcmp(old_paw2,paw_2) != 0)
	    {
		hmdir_(paw_2,"S",strlen(paw_2),1);
		strcpy(old_paw2,paw_2);   	                                
	    }
	    else
		hcdir_(paw_2," ",strlen(paw_2),1);
	}              /* end of loop for str_num = 2     */
	if(str_num == 3)
	{                 /* start of loop for str_num = 3   */
	    if(strcmp(old_paw0,paw_0) != 0)
	    {
		hmdir_(paw_0, "S", strlen(paw_0), 1);
		strcpy(old_paw0,paw_0);
	    }
	    else
		hcdir_(paw_0," ",strlen(paw_0),1);
      
	    if(strcmp(old_paw1,paw_1) !=0)
	    {
		hmdir_(paw_1,"S",strlen(paw_1),1);
		strcpy(old_paw1,paw_1);
	    }
	    else
		hcdir_(paw_1," ",strlen(paw_1),1);
       
	    if(strcmp(old_paw2,paw_2) != 0)
	    {
		hmdir_(paw_2,"S",strlen(paw_2),1);
		strcpy(old_paw2,paw_2);   	                                
	    }
	    else
		hcdir_(paw_2," ",strlen(paw_2),1);
         
              
	    if(strcmp(old_paw3,paw_3) != 0)
	    {
		hmdir_(paw_3,"S",strlen(paw_3),1);
		strcpy(old_paw3,paw_3);   	                                
	    }
	    else
		hcdir_(paw_3," ",strlen(paw_3),1);
	}              /* end of loop for str_num = 3    */
               
      
         
	/*     hldir_("//PAWC","T",6,1);          */   
     
 
      
	if (data_type == HS_1D_HISTOGRAM)    
	{                                 /*  start of 1 d Histo     */
	    ++nh1;
	    num_bins = hs_1d_hist_num_bins(ids[i]);
         
        
	    ptd_data       = (float *) malloc(num_bins * sizeof (float));
	    ptd_pos_errors = (float *) malloc(num_bins * sizeof (float));
	    ptd_neg_errors = (float *) malloc(num_bins * sizeof (float));
	    if (ptd_data == NULL || ptd_pos_errors == NULL || 
		ptd_neg_errors == NULL)
	    {
		fprintf(stderr, "Fatal error: out of memory. Exiting.\n");
		exit(EXIT_FAILURE);
	    }

	    hs_1d_hist_bin_contents(ids[i], ptd_data);
	    status=hs_1d_hist_errors(ids[i], ptd_pos_errors, ptd_neg_errors);
         
#ifdef H_DEBUG
	    fprintf(stderr,"This is a 1 d histo \n");
	    fprintf(stderr,"The number of bins in this 1-d histo are: %d \n",
		    num_bins);
	    fprintf(stderr,"This is the 1 2 and last data point %f %f %f \n",
		    ptd_data[1],ptd_data[2],ptd_data[num_bins]);
	    fprintf(stderr,"The status for hs_1d_hist errors is %d \n",status);
         
	    if (status == 1 & (i%20 == 0))
	    {
		for (j=0 ; j<num_bins; j++)
		{
		    fprintf(stderr,"This is bin j %d it's value is and error %f %f \n"
			    ,j,ptd_data[j],ptd_pos_errors[j]);
		}
	    }
#endif    /* H_DEBUG   */
         
	    hs_1d_hist_range(ids[i], &min, &max);
	    min_u = min - ( (max - min) / ((float)num_bins));
	    max_o = max + ( (max - min) / ((float)num_bins));
	    hs_1d_hist_overflows(ids[i], &underflow, &overflow);
         
#ifdef H_DEBUG
	    fprintf(stderr,"1d min %f max %f \n",min,max);
	    fprintf(stderr,"1d underflow %f overflow %f \n", underflow,overflow);
#endif    /* H_DEBUG */
         
	    hbook1_(&id_user,title_string,&num_bins,&min,&max,
		    &con_0,length1);
	    hpak_(&id_user,ptd_data);
        
	    if((status == HS_POS_ERRORS) || (status == HS_BOTH_ERRORS))
		hpake_(&id_user,ptd_pos_errors);
                
	    if(underflow > 0.)
		hfill_(&id_user,&min_u,&con_0,&underflow);
	    if(overflow > 0.)
		hfill_(&id_user,&max_o,&con_0,&overflow);
        
        
	    if(i%20 ==0)     /* We will print out only every 20 th histogram */
	    {                                
		for (j=0; j<num_bins; j=j+5)                  
		{
#ifdef H_DEBUG                                          
		    fprintf(stderr," The value of i and ids[i] %d %d and j and data %d"
			    "%f \n", i, id_user, j,ptd_data[j]);
#endif   /*   H_DEBUG   */
		}
	    }
	    free(ptd_data);
	    free(ptd_pos_errors);
	    free(ptd_neg_errors);
            
         
	}                            /* end of loop over 1d histo    */
	else if (data_type == HS_2D_HISTOGRAM)
	{
	    ++nh2;
	    hs_2d_hist_num_bins(ids[i],&num_x_bins,&num_y_bins);
	    hs_2d_hist_stats(ids[i],&x_mean,&y_mean,&x_std_dev,&y_std_dev);
          
          
	    ptd_2d_data      = 
		(float *) malloc(num_x_bins*num_y_bins * sizeof(float));
	    ptd_2d_pos_errors =
		(float *) malloc(num_x_bins*num_y_bins * sizeof(float));
	    ptd_2d_neg_errors =
		(float *) malloc(num_x_bins*num_y_bins * sizeof(float));
	    ptd_2di_data      =
		(float *) malloc(num_y_bins*num_x_bins * sizeof(float));
	    ptd_2d_errors     =
		(float *) malloc(num_x_bins*num_y_bins * sizeof(float));
	    if (ptd_2d_data == NULL || ptd_2d_pos_errors == NULL ||
		ptd_2d_neg_errors == NULL || ptd_2di_data == NULL ||
		ptd_2d_errors == NULL)
	    {
		fprintf(stderr, "Fatal error: out of memory. Exiting.\n");
		exit(EXIT_FAILURE);
	    }
          
	    hs_2d_hist_bin_contents(ids[i],ptd_2d_data);
	    status =
		hs_2d_hist_errors(ids[i],ptd_2d_pos_errors,ptd_2d_neg_errors);
	    hs_2d_hist_range(ids[i], &x_min, &x_max, &y_min, &y_max);
	    hs_2d_hist_overflows(ids[i],overflows);
	    x_min_u = x_min - ( (x_max - x_min) / ((float)num_x_bins));
	    x_max_o = x_max + ( (x_max - x_min) / ((float)num_x_bins));
	    y_min_u = y_min - ( (y_max - y_min) / ((float)num_y_bins));
	    y_max_o = y_max + ( (y_max - y_min) / ((float)num_y_bins));
          
#ifdef H_DEBUG
	    fprintf(stderr,"This is a 2 dim Histogram \n");
	    fprintf(stderr,"The number of x and y bins in this 2 d histo"
		    " are %d and %d \n", num_x_bins, num_y_bins);
	    fprintf(stderr,"2d  x_min %f x_max %f y_min %f y_max %f \n"
		    , x_min, x_max, y_min, y_max);
	    fprintf(stderr,"The status of hs_2d_hist errors is %d \n",status);
	    fprintf(stderr,"The 2d overflows first row are %f %f %f \n" 
		    ,overflows[0][0], overflows[0][1], overflows[0][2]);
	    fprintf(stderr,"The 2d overflows of the second row are %f %f %f \n"
		    ,overflows[1][0],overflows[1][1],overflows[1][2]);
	    fprintf(stderr,"The 2d overflows of the third row are %f %f %f \n" 
		    ,overflows[2][0],overflows[2][1],overflows[2][2]);
#endif    /* H_DEBUG */
          
	    hbook2_(&id_user,title_string,&num_x_bins,&x_min,&x_max,
		    &num_y_bins,&y_min,&y_max,&con_0,length1);
	    for  (k1 = 0; k1 < num_x_bins; k1++)
		for (k2 = 0; k2 < num_y_bins; k2++)
		{
		    ptd_2di_data[k2*num_x_bins + k1] = ptd_2d_data[k1*num_y_bins + k2];
		    if((status == HS_POS_ERRORS) || (status == HS_BOTH_ERRORS))
			ptd_2d_errors[k2*num_x_bins + k1] = 
			    ptd_2d_pos_errors[k1*num_y_bins + k2];
		}
        
	    hpak_(&id_user,ptd_2di_data);
	    if((status == HS_POS_ERRORS) || (status == HS_BOTH_ERRORS))
		hpake_(&id_user,ptd_2d_errors);
	    if(overflows[0][0] > 0.)
		hfill_(&id_user,&x_min_u,&y_max_o,&overflows[0][0]);
	    if(overflows[0][1] > 0.)
		hfill_(&id_user,&x_mean,&y_max_o,&overflows[0][1]);
	    if(overflows[0][2] > 0.)
		hfill_(&id_user,&x_max_o,&y_max_o,&overflows[0][2]);
	    if(overflows[1][0] > 0.)
		hfill_(&id_user,&x_min_u,&y_mean,&overflows[1][0]);
	    if(overflows[1][2] > 0.)
		hfill_(&id_user,&x_max_o,&y_mean,&overflows[1][2]);
	    if(overflows[2][0] > 0.)
		hfill_(&id_user,&x_min_u,&y_min_u,&overflows[2][0]);
	    if(overflows[2][1] > 0.)
		hfill_(&id_user,&x_mean,&y_min_u,&overflows[2][1]);
	    if(overflows[2][2] >0.)
		hfill_(&id_user,&x_max_o,&y_min_u,&overflows[2][2]); 
	    free(ptd_2d_data);
	    free(ptd_2di_data);
	    free(ptd_2d_pos_errors);
	    free(ptd_2d_neg_errors);
	    free(ptd_2d_errors);        
    
                        
	}
	else if (data_type == HS_NTUPLE)
	{
	    ++nnt;
	    num_entries = hs_num_entries(ids[i]);
	    num_var = hs_num_variables(ids[i]);
          
	    variable_name =  (char *) malloc(num_var *8* sizeof(char));
	    variable_n    =  (char *) malloc(num_var *8* sizeof(char));
	    variable_length = (int *) malloc(num_var * sizeof(int));
	    ptd_ntuple_row = (float*) malloc(num_var * sizeof(float));
	    if (variable_name == NULL || variable_n == NULL || 
		variable_length == NULL || ptd_ntuple_row == NULL)
	    {
		fprintf(stderr, "Fatal error: out of memory. Exiting.\n");
		exit(EXIT_FAILURE);
	    }
          
#ifdef H_DEBUG
	    fprintf(stderr,"This is an ntuple \n");
	    fprintf(stderr,"The number of variables in this ntuple is %d \n",
		    num_var);
	    fprintf(stderr,"The number of entries (rows) in this ntuple %d\n",
		    num_entries);
#endif   /*   H_DEBUG   */        
          
	    for (l=0; l<num_var; l++)
	    {
		length_n = hs_variable_name(ids[i], l,&variable_name[l]);
		variable_length[l] = length_n;
          
#ifdef H_DEBUG
		fprintf(stderr,"The number of characters in this name is %d \n",
			length_n);
		fprintf(stderr,"the number of characters in this name is %d \n",
			variable_length[l]);
		fprintf(stderr,"The %d name (variable) in this ntuple is  \n",l);
#endif   /*   H_DEBUG   */

		l2 = 0;
		for (l1=l; l1<l +  8 ;l1++,l2++)
		{
		    if (l1 < l + length_n)
		    {
			variable_n[l*8 + l2] = variable_name[l1];
            
#ifdef H_DEBUG
			fprintf(stderr,"%c",variable_name[l1]);
			fprintf(stderr," \n");
			fprintf(stderr," index is %d the character is %c \n",l*8 +l2,
				variable_n[l*8 +l2]);
#endif    /*   H_DEBUG   */
            
		    }
		    else
		    {
			variable_n[l*8 + l2] = ' ';
		    }
		}
	    }
          
#ifdef H_DEBUG
	    fprintf(stderr," The variable name is %64s \n",variable_n);
	    fprintf(stderr, "before hbookn id_user %d \n",id_user);
	    fprintf(stderr, "before hbookn &id_user %p \n",&id_user);
#endif     /*   H_DEBUG   */
	    hbookn_(&id_user,title_string,&num_var," ",&nwords,
		    variable_n,length1,1,8);
	    for(l3=0; l3<num_entries; l3++)
	    {
		if(l3 == 0){
#ifdef H_DEBUG
		    fprintf(stderr," id_user is %d \n",id_user);
		    fprintf(stderr," &id_user is %p \n",&id_user);
#endif   /*   H_DEBUG   */
		}
		hs_row_contents(ids[i],l3,ptd_ntuple_row);
		hfn_(&id_user,ptd_ntuple_row);
	    }
	    free(ptd_ntuple_row);  
	    free(variable_length);
	    free(variable_n);
	    free(variable_name);
	}  
	else 
	    fprintf(stderr,"this is not a 1 d histo  or a 2 d histo"
		    " or an ntuple \n");

	hs_delete(ids[i]);
    }                        /* end of loop over items      */
     
    hs_complete();
         
    /* hprint_(&con_0);  */         /* print out all histograms  */

    tohbf_(file_hbook, &LREC, strlen(file_hbook));

/*     fprintf(stderr,"The Hbook file %s is now complete\n",file_hbook); */
    printf("\n The Hbook file %s in now complete.\n", file_hbook);
    printf( " The output contains %d 1-d histogram(s), %d 2-d histogram(s),"
	    " and %d ntuple(s).\n",nh1,nh2,nnt);
    exit(EXIT_SUCCESS);
}
