/*******************************************************************************
*									       *
* GetHbookData.c -- Small and minimal interface between a RZ HBOOK file and    *
* the HistoScope Server Process.  Given a UNIX filename, this routine will fill*
* the Data Field of the Histo.h structure.                                     * 
*									       *
* Copyright (c) 1991, 1993 Universities Research Association, Inc.	       *
* All rights reserved.							       *
* 									       *
* This material resulted from work developed under a Government Contract and   *
* is subject to the following license:  The Government retains a paid-up,      *
* nonexclusive, irrevocable worldwide license to reproduce, prepare derivative *
* works, perform publicly and display publicly by or for the Government,       *
* including the right to distribute to other Government contractors.  Neither  *
* the United States nor the United States Department of Energy, nor any of     *
* their employees, makes any warrenty, express or implied, or assumes any      *
* legal liability or responsibility for the accuracy, completeness, or         *
* usefulness of any information, apparatus, product, or process disclosed, or  *
* represents that its use would not infringe privately owned rights.           *
*                                        				       *
* Fermilab Nirvana GUI Library						       *
* April 28, 1992							       *
*									       *
* Written by Paul Lebrun						       *
* August 12 1992 : Adding hbneedtoupdate.				       *
* September 3, 1993 : Adding error retrieval to getHbookData(F) - JMK.         *
*		      new getErrFlg parameter to GetHbookData()                *
*									       *
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "histo.h"
#include "getHbookData.h"

#ifdef VMS

#include "../util/vmsUtils.h"

#define closehbookfile_ closehbookfile
#define hrin_ hrin
#define hdelet_ hdelet
#define hgnf_ hgnf
#define hi_ hi
#define hij_ hij
#define hnoent_ hnoent
#define hgnpar_ hgnpar
#define hexist_ hexist
#endif /*VMS*/

/* Function Prototypes for utilHbook.f & hbook routines */
#ifdef VMS
extern int openhbookfile(struct dsc$descriptor_s *fname, int *lrec);
extern void hcdir(struct dsc$descriptor_s *dn, struct dsc$descriptor_s *dm);
static void hcdir_(char *dn, char *dm, int l1, int l2);
static void hcdirw_(char *dn, char *dm, int l1, int l2);
extern void hunpak(int *i1, float *a, struct dsc$descriptor_s *fn, int *i2);
extern void hunpke(int *i1, float *a, struct dsc$descriptor_s *fn, int *i2);
#else
extern int openhbookfile_(char *fname, int *lrec, int l);
extern void hcdir_(char *dn, char *dm, int l1, int l2);
#define hcdirw_ hcdir_
extern void hunpak_(int *i1, float *a, char *fn, int *i2, int l1);
extern void hunpke_(int *i1, float *a, char *fn, int *i2, int l1);
#endif /*VMS*/
extern void hnoent_(int *i1, int *i2);
extern void closehbookfile_();
extern void hrin_(int *i1, int *i2, int *i3);
extern void hdelet_(int *i);
extern void hgnf_(int *i1, int *i2, float *a, int *i3); 
extern float hi_(int *i1, int *i2); 
extern float hij_(int *i1, int *i2, int *i3);

int GetHbookDataF(char* filename, int lrec, hsGeneral *hsg)
{
	char c_space[8], category[HS_MAX_CATEGORY_LENGTH+1];
	int i, l, istat, n1, n2;
	hsNTuple * hnt;
/*
**	Open the RZ file, which contains Histogram Data, and make a 
**	listing of all histograms
*/ 
#ifdef VMS
	struct dsc$descriptor_s *ds, ds1;

	if (openhbookfile((ds = NulStrToDesc(filename)), &lrec) != 0) {
	    FreeStrDesc(ds);
	    return HBOOK_DATA_FILE_OPEN_ERR;
	}
	FreeStrDesc(ds);
#else
	if (openhbookfile_(filename, &lrec, strlen(filename)) != 0)
	    return HBOOK_DATA_FILE_OPEN_ERR;
#endif /*VMS*/
/*
**	Set default to the appropriate directory.
*/
	strcpy(c_space, "   ");
	sprintf(category, "//%s", hsg->category);
	hcdir_(category, c_space, strlen(category), strlen(c_space));
	l = 0; i = 999999;
	hrin_(&hsg->hbookID, &i, &l);
	if (hsg->type == HS_NTUPLE) {
	   hnt = (hsNTuple * ) hsg;
	   hnoent_(&hnt->hbookID, &hnt->n);
	   n1 = 0;
	   n2 = hnt->n;
	}
	else {
	   n1 = 0; n2 = 1;
	}
	istat = GetHbookData(hsg, n1, n2, 1); 	/* 1 = ask for errors */
	hdelet_(&hsg->hbookID);
	closehbookfile_();
	return istat;   
}

int GetHbookData(hsGeneral *hsg, int fromTuple, int toTuple, int getErrFlg)
{ 
	int i, j, k, l, nb1, nb2, num, ierr, ievt, nsp, itmp;
	float *d2, *dataPtr;
	hs1DHist *hh1;
	hs2DHist *hh2;
	hsNTuple *hnt;
	char c_space[8], category[HS_MAX_CATEGORY_LENGTH+1];
	char saveCat[HS_MAX_CATEGORY_LENGTH+1];
#ifdef VMS
	struct dsc$descriptor_s *ds;
#endif /*VMS*/

/*
**	Set default to the appropriate directory.  In case of File interface,
**	this is a waste of time, as we already are in the correct Hbook 
**	directory, but what the hell, it does not hurt.  In the client case,
**	we must save the current directory and restore it later.
*/
	 strcpy(c_space, "   ");
	 memset(saveCat, 0, HS_MAX_CATEGORY_LENGTH+1);
	 hcdirw_(saveCat, "R", HS_MAX_CATEGORY_LENGTH, 1);
	 sprintf(category, "//%s", hsg->category);
	 hcdir_(category, c_space, strlen(category), 4);
	 if ( hsg->type == HS_1D_HISTOGRAM) {
/*
**	One dimension histogram
*/
	  hh1 = (hs1DHist *) hsg;
	  nb1 = hh1->nBins;
	  hh1->bins = (float *) malloc(sizeof(float)*nb1);
	  if (hh1->bins == NULL) return HBOOK_DATA_TOO_BIG;
	  strcpy(c_space,"HIST");
	  num = 0;
#ifdef VMS
	  hunpak(&hsg->hbookID, hh1->bins, (ds = NulStrToDesc(c_space)), &num);
#else
	  hunpak_(&hsg->hbookID, hh1->bins, c_space, &num, 4);
#endif /*VMS*/
	  hnoent_(&hsg->hbookID, &hh1->count);
	  i = 0;
	  hh1->underflow = hi_(&hsg->hbookID, &i);
	  i = nb1 + 1;
	  hh1->overflow = hi_(&hsg->hbookID, &i);
	  hh1->errFlg = 1;	       /* HBOOK always gives errors upon req. */
	  hh1->mErrs = NULL;	       /* (-) errors will always be = NULL    */
	  if (getErrFlg) {	       /* get errors now?		      */
	      hh1->pErrs = (float *) malloc(sizeof(float)*nb1);	/* yes	      */
	      if (hh1->pErrs == NULL) {
	          hh1->errFlg = 0;
#ifdef VMS
	          FreeStrDesc(ds);
#endif /*VMS*/
	          return HBOOK_ERRORS_TOO_BIG;
	      }
#ifdef VMS
	      hunpke(&hsg->hbookID, hh1->pErrs, ds, &num);
#else
	      hunpke_(&hsg->hbookID, hh1->pErrs, c_space, &num, 4);
#endif /*VMS*/
#ifdef COM_DEBUG
	      if (hh1->pErrs != NULL) {
		  printf("Errors:\t");
		  for (itmp = 0; itmp < hh1->nBins; ++itmp)
	      	      printf("%f\t", hh1->pErrs[itmp]);
		  printf("End:\n");
	      } 
#endif /*COM_DEBUG*/
	  }
	  else
	      hh1->pErrs = NULL;       /* (+) errors			      */
#ifdef VMS
	  FreeStrDesc(ds);
#endif /*VMS*/
	}
	 if ( hsg->type == HS_2D_HISTOGRAM) {
/*
**	Two dimensions histogram
*/
	  hh2 = (hs2DHist *) hsg;
	  nb1 = hh2->nXBins;
	  nb2 = hh2->nYBins;
	  hh2->bins = (float *) malloc(sizeof(float)*nb1*nb2);
	  if (hh2->bins == NULL) return HBOOK_DATA_TOO_BIG;
	  d2 = (float *) malloc(sizeof(float)*nb1*nb2);
	  if (d2 == NULL) {
	  	 free (hh2->bins);
	  	 hh2->bins = NULL;
	  	 return HBOOK_DATA_TOO_BIG;
	  }
	  strcpy(c_space,"HIST");
	  num = 0;
#ifdef VMS
	  hunpak(&hsg->hbookID, d2, (ds = NulStrToDesc(c_space)), &num);
#else
	  hunpak_(&hsg->hbookID, d2, c_space, &num, 6);
#endif /*VMS*/
	  hnoent_(&hsg->hbookID, &hh2->count);
	  for (i=0; i<nb1; i++) {
	      for (j=0; j<nb2; j++){
	          k = i*nb2 +j;
	          l = j*nb1 +i;
	          *(hh2->bins+k) = *(d2+l);
	      }
	   }
	   hh2->errFlg = 1;	       /* HBOOK always gives errors upon req. */
	   hh2->mErrs = NULL;	       /* (-) errors will always be = NULL    */
	   if (getErrFlg) {	       /* get errors now?		      */
	      hh2->pErrs = (float *) malloc(sizeof(float)*nb1*nb2); /* yes    */
	      if (hh2->pErrs == NULL) {
	          hh2->errFlg = 0;
	          free (d2);
#ifdef VMS
	          FreeStrDesc(ds);
#endif /*VMS*/
	          return HBOOK_ERRORS_TOO_BIG;
	      }
#ifdef VMS
	      hunpke(&hsg->hbookID, d2, ds, &num);
#else
	      hunpke_(&hsg->hbookID, d2, c_space, &num, 4);
#endif /*VMS*/
	      for (i=0; i<nb1; i++) {	 /* get from col maj to row maj order */
	          for (j=0; j<nb2; j++){
	              k = i*nb2 +j;
	              l = j*nb1 +i;
	              *(hh2->pErrs+k) = *(d2+l);
	          }
	      }
#ifdef COM_DEBUG
	      if (hh2->pErrs != NULL) {
		 printf("Errors:\t");
		 for (itmp = 0; itmp < hh2->nXBins && itmp < hh2->nYBins; ++itmp)
	      	     printf("%f\t", hh2->pErrs[itmp + itmp*hh2->nXBins]);
		 printf("End:\n");
	      } 
#endif /*COM_DEBUG*/
	   }
	   else
	      hh2->pErrs = NULL;       /* (+) errors			      */
#ifdef VMS
	   FreeStrDesc(ds);
#endif /*VMS*/
	   free (d2);
	   i = 0;
	   j = nb1 + 1;
	   k = nb2 + 1;
	   hh2->overflow[0][0] = hij_(&hsg->hbookID, &i, &k);  /* -x +y */
	   hh2->overflow[0][2] = hij_(&hsg->hbookID, &i, &i);  /* -x -y */
	   hh2->overflow[2][0] = hij_(&hsg->hbookID, &j, &k);  /* +x +y */
	   hh2->overflow[2][2] = hij_(&hsg->hbookID, &j, &i);  /* +x -y */
	   hh2->overflow[0][1] = 0;				/* -x   */
	   hh2->overflow[2][1] = 0;				/* +x   */
	   hh2->overflow[1][2] = 0;				/* -y   */
	   hh2->overflow[1][0] = 0;				/* +y   */
	   for (l = 1; l <= nb2; ++l) {
	       hh2->overflow[0][1] += hij_(&hsg->hbookID, &i, &l); /* -x */
	       hh2->overflow[2][1] += hij_(&hsg->hbookID, &j, &l); /* +x */
	   }
	   for (l = 1; l <= nb1; ++l) {
	       hh2->overflow[1][2] += hij_(&hsg->hbookID, &l, &i); /* -y */
	       hh2->overflow[1][0] += hij_(&hsg->hbookID, &l, &k); /* +y */
	   }
	   
	}
	if ( hsg->type == HS_NTUPLE) {
/*
**	NTuple.
*/
	  hnt = (hsNTuple *) hsg;
	  nb1 = hnt->nVariables;
	  hnt->n = toTuple;
/*
** Alocate space, check if not too big.
*/
	  if (hnt->n == 0) return HBOOK_DATA_MISSING;
	  if (hnt->data != NULL) free(hnt->data);
	  nsp = sizeof(float) * (toTuple - fromTuple) * nb1;
	  hnt->data = (float *) malloc(nsp);
	  if (hnt->data == NULL) {
	    hnt-> n = 0;
	    return HBOOK_DATA_TOO_BIG;
	  }
	  hnt->chunkSzData = toTuple - fromTuple; hnt->chunkSzExt = 10;
	  for (ievt = 0; ievt < 4; ++ievt)
	      hnt->extensions[ievt] = NULL;
	  dataPtr = hnt->data;
	  for (ievt=fromTuple+1; ievt<= toTuple; ievt++) {
	    /* Bypass HBOOK 4 bug in subroutine hgnf,
	       this call should not be necessary */
	    hgnpar_(&hsg->hbookID, "HGNF", 4);
	    hgnf_(&hsg->hbookID, &ievt, dataPtr,  &ierr);
	    if (ierr != 0) {
	      hnt->n = ievt - 1;
	      return HBOOK_DATA_CORRUPTED;
	      }
	    dataPtr += nb1;
	  }
	}
	/* set directory back to where user had it */
	hcdir_(saveCat,  c_space, strlen(saveCat), 4);
	return HBOOK_DATA_OK;
}


      int hbneedtoupdate( hsGeneral* hsg, int updateLevel )
{ 
	extern int hexist_(int *i);
/*
** Code
*/
	char  c_space[8], category[HS_MAX_CATEGORY_LENGTH+1];
	int i, ihn, numhb, numToRtn = 0, ievt, ierr;
	float *tf, *first_row;
	char saveCat[HS_MAX_CATEGORY_LENGTH+1];
	hsNTuple *hnt;

	    strcpy(c_space, "   ");
	    memset(saveCat, 0, HS_MAX_CATEGORY_LENGTH+1);
	    hcdirw_(saveCat, "R", HS_MAX_CATEGORY_LENGTH, 1);
	    sprintf(category, "//%s", hsg->category);
	    hcdir_(category, c_space, strlen(category), 4);
	    ihn = hsg->hbookID;
	    if (hexist_(&ihn) != 0) {
	      hnoent_(&ihn, &numhb);
	      if (hsg->type == HS_NTUPLE) {
	         hnt = (hsNTuple *) hsg;
		 if (numhb > updateLevel) numToRtn = numhb;
/*
** Check for the possibility of having a circular buffer. e.g. a fixed 
** number of entries, but changing data. 
*/
		 if ((numhb == updateLevel) && (numhb != 0)) {
		      first_row = (float *)
		       malloc(sizeof(float)*hnt->nVariables);
	               hgnpar_(&ihn, "HGNF", 4);
	               ievt = 1;
	               hgnf_(&ihn, &ievt, first_row,  &ierr);
	               if (ierr == 0) { 
	                 tf = first_row;
	                 for (i=0; i<hnt->nVariables; i++, tf++ )
	                    if (hnt->data != NULL && *tf != hnt->data[i])
	                        numToRtn = HBOOK_NTUPLE_IS_CIRCULAR ;
	                }
	               free(first_row);
	               }
	      }   
	      else if (numhb != updateLevel) numToRtn = 1;
	    }
	    /* set directory back to where user had it */
	    hcdir_(saveCat,  c_space, strlen(saveCat), 4);
	    return numToRtn;   
}
   
#ifdef VMS

static void hcdir_(char *dn, char *dm, int l1, int l2)
{
    struct dsc$descriptor *ds1, *ds2;

    hcdir((ds1 = NulStrToDesc(dn)), (ds2 = NulStrToDesc(dm)));
    FreeStrDesc(ds1);
    FreeStrDesc(ds2);
}

static void hcdirw_(char *dn, char *dm, int l1, int l2)
{
    struct dsc$descriptor *ds1, *ds2;

    ds2 = NulStrToDesc(dm);
    ds1 = NulStrWrtDesc(dn, l1);
    hcdir(ds1, ds2);
    FreeStrDesc(ds1);
    FreeStrDesc(ds2);
}
#endif /*VMS*/
