/*******************************************************************************
*									       *
* scandir.c -- Routine to substitute for unix scandir on VMS and Solaris       *
*	       systems for the VMS histoscope tool			       *
*									       *
* Copyright (c) 1994 Universities Research Association, Inc.		       *
* All rights reserved.							       *
* 									       *
* This material resulted from work developed under a Government Contract and   *
* is subject to the following license:  The Government retains a paid-up,      *
* nonexclusive, irrevocable worldwide license to reproduce, prepare derivative *
* works, perform publicly and display publicly by or for the Government,       *
* including the right to distribute to other Government contractors.  Neither  *
* the United States nor the United States Department of Energy, nor any of     *
* their employees, makes any warrenty, express or implied, or assumes any      *
* legal liability or responsibility for the accuracy, completeness, or         *
* usefulness of any information, apparatus, product, or process disclosed, or  *
* represents that its use would not infringe privately owned rights.           *
*                                        				       *
* Fermilab Nirvana GUI Library						       *
* April 1, 1993								       *
*									       *
* Written by Joy Kyriakopulos						       *
*									       *
* updated for Solaris 10/11/94 - JMK
*									       *
*******************************************************************************/
#ifdef VMS
#include <unixio.h>
#endif /*VMS*/
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#ifdef VMS
#include <fab.h>
#include <nam.h>
#include <rmsdef.h>
#else
#include <sys/types.h>
#include <dirent.h>
#include <sys/param.h>
#endif /*VMS*/
#include "scandir.h"

#define MAX_NUM_FILES 100

#ifdef VMS
static void successRtn(struct FAB *dirFab);
static void errRtn(struct FAB *dirFab);
static char **Namelist;

static int NumFilesFound;
static int (*SelectRoutine)();		/* saves select fcn for successRtn    */

int scandir(char *dirname, char *(*namelist[]), int (*select)(), void *compar)
{
    struct FAB dirFAB;
    struct NAM dirNAM;
    char result_name[NAM$C_MAXRSS+1];	/* array for resulting file spec      */
    char expanded_name[NAM$C_MAXRSS+1];	/* array for expanded file spec	      */
    int stat;

    dirFAB = cc$rms_fab;		/* initialize FAB with default values */
    dirNAM = cc$rms_nam;		/*     "      NAMe block "  "   "     */
    dirFAB.fab$l_nam = &dirNAM;		/* point FAB to NAM block	      */
    dirFAB.fab$l_fna = dirname;		/* wildcard spec for LIB$FILE_SCAN    */
    dirFAB.fab$b_fns = strlen(dirname);
    dirFAB.fab$l_dna = "*.";		/* default is no extension            */
    dirFAB.fab$b_dns = 2;
    dirNAM.nam$l_esa = &expanded_name;	/* expanded file specs returned here  */
    dirNAM.nam$b_ess = sizeof(expanded_name) - 1;
    dirNAM.nam$l_rsa = &result_name;	/* resulting file specs returned here */
    dirNAM.nam$b_rss = sizeof(result_name) - 1;

    SelectRoutine = select;
    NumFilesFound = 0;
    Namelist = malloc(sizeof(char *) * MAX_NUM_FILES);
    *namelist = 0;
    stat = LIB$FILE_SCAN(&dirFAB, successRtn, errRtn, 0); 

    if (stat != RMS$_NORMAL && stat != RMS$_FNF && stat != RMS$_NMF) {
	fprintf(stderr, "Error calling LIB$FILE_SCAN: %s\n",
		strerror(EVMSERR, stat));
	return -1;
    }
    if (stat == RMS$_FNF)		/* file not found		      */
	return 0;
    stat = LIB$FILE_SCAN_END(&dirFAB, 0);
    *namelist = Namelist;
    return NumFilesFound;
}

static void successRtn(struct FAB *dirFab)
{
    if (NumFilesFound >= MAX_NUM_FILES)
	return;

    /* terminate filename string with a null to pass to user's select routine */
    dirFab->fab$l_nam->nam$l_rsa[dirFab->fab$l_nam->nam$b_rsl] = '\0';

    /* if user's select routine returns value != 0, then put into name list */
    if (SelectRoutine == NULL || 
		(*SelectRoutine)(dirFab->fab$l_nam->nam$l_rsa)) {
	++NumFilesFound;
	Namelist[NumFilesFound-1] = malloc(dirFab->fab$l_nam->nam$b_rsl+1);
	strcpy(Namelist[NumFilesFound-1], dirFab->fab$l_nam->nam$l_rsa);
    }
}

static void errRtn(struct FAB *dirFab)
{
    if (dirFab->fab$l_sts != RMS$_FNF) {
	fprintf(stderr, "Error detected by LIB$FILE_SCAN:  ");
	fprintf(stderr, "%s.  (%d)\n", strerror(EVMSERR, dirFab->fab$l_sts),
		dirFab->fab$l_sts);
    }
}

#else  /*Solaris below*/

int scandir(char *dirname, char **namelist[], int (*select)(char *dir),  
	void *compar)
{
    DIR *dp;
    struct dirent *dirp;
    int len, numFilesFound = 0;
    char **nameList = 0;
    char fileName[MAXPATHLEN];
    
    if ( (dp = opendir(dirname)) == NULL )  {
    	fprintf(stderr, "hs_initialize: Cannot open directory %s\n", dirname);
    	return numFilesFound;
    }
    *namelist = 0;
    nameList = malloc(sizeof(char *) * MAX_NUM_FILES);
    if (nameList == NULL)  {
    	fprintf(stderr, "hs_initialize: Cannot do malloc (in scandir)\n");
    	return numFilesFound;
    }
#ifdef COM_DEBUG
    printf("\nin scandir, scanning directory: %s", dirname);
#endif /*COM_DEBUG*/
    strcpy(fileName, dirname);
    strcat(fileName, "/");
    len = strlen(fileName);
    while ( (dirp = readdir(dp)) != NULL )  {
#ifdef COM_DEBUG
    	printf("\n            found file: %s\n", dirp->d_name);
#endif /*COM_DEBUG*/
    	strcpy(fileName+len, dirp->d_name);
    	if ( select == NULL || (*select)(dirp->d_name) ) {
	    nameList[numFilesFound] = malloc(strlen(fileName)+1);
	    strcpy(nameList[numFilesFound], fileName);
	    ++numFilesFound;
#ifdef COM_DEBUG
    	    printf(" ** SELECTED ** ");
#endif /*COM_DEBUG*/
    	    if (numFilesFound >= MAX_NUM_FILES) {
		break;
	    }
	}
    }
#ifdef COM_DEBUG
    printf("\n    %d files selected.\n", numFilesFound);
#endif /*COM_DEBUG*/
    if (numFilesFound > 0)
	*namelist = nameList;
    else
    	free(nameList);
    closedir(dp);
    return numFilesFound;
}

#endif /*VMS*/
