#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include <X11/Intrinsic.h>
#include <Xm/Xm.h>
#include <Xm/PushB.h>
#include <Xm/ScrollBar.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/ToggleB.h>
#include <Xm/RowColumn.h>
#include <Xm/Separator.h>
#if XmVersion == 1002
#include <Xm/PrimitiveP.h>
#endif
#include "CurvesP.h"
#include "../util/misc.h"
#include "../util/printUtils.h"

#define MAX_DISP_VARS 14

typedef struct {
    Widget shell;
    Widget widget;
    Widget menu;
    int ntVars[MAX_DISP_VARS];
} windowInfo;

typedef struct {
    int x;
} hsGeneral;


static int readData(CurveStruct **curves);
static void redisplayTS(windowInfo *window, hsGeneral *item, int reInit);
static void createPlotMenu(windowInfo *wInfo);
static void popupCB(Widget w, Widget menu, XButtonEvent *event);
static Widget addMenuItem(Widget parent, char *name, char *label,
			  char mnemonic, char *acc, char *accText,
			  XtCallbackProc callback, void *cbArg);
static Widget addMenuToggle(Widget parent, char *name, char *label,
		 	    char mnemonic, char *acc, char *accText,
		  	    XtCallbackProc callback, void *cbArg, int set);
static Widget addMenuSeparator(Widget parent, char *name);
static void closeCB(Widget w, windowInfo *wInfo, caddr_t callData);
static void showRangeCB(Widget w, windowInfo *wInfo, caddr_t callData);
static void toggleXLogCB(Widget w, windowInfo *wInfo, caddr_t callData);
static void toggleYLogCB(Widget w, windowInfo *wInfo, caddr_t callData);
static void showSlidersCB(Widget w, windowInfo *wInfo, caddr_t callData);
static void redisplayXY(windowInfo *window, hsGeneral *item, int reInit);
static void curvesZoomInCB(Widget w, windowInfo *wInfo, caddr_t callData);
static void curvesZoomOutCB(Widget w, windowInfo *wInfo, caddr_t callData);
static void curvesResetZoomCB(Widget w, windowInfo *wInfo, caddr_t callData);
static void curvesSetMarkLineStyleCB(Widget w,windowInfo *wInfo,caddr_t callData);
static void curvesScaleVariablesCB(Widget w, windowInfo *wInfo, caddr_t callData);
static void curvesShowLegendCB(Widget w,windowInfo *wInfo,caddr_t callData);
static Boolean showLegend(windowInfo *wInfo, Boolean isTimeSeries); 
static void showSlidersCB(Widget w, windowInfo *wInfo, caddr_t callData);
static void scatStatsCB(Widget w, windowInfo *wInfo, caddr_t callData);
static void scatResetCB(Widget w, windowInfo *wInfo, caddr_t callData);
static void curvesPrintCB(Widget w, windowInfo *wInfo, caddr_t callData);
static void scatAxisCB(Widget w, windowInfo *wInfo, caddr_t callData);
void CreateTSPlotMenu(windowInfo *wInfo);
void CreateXYPlotMenu(windowInfo *wInfo);

int nCurves;
char xLabel[80], yLabel[80];
CurveStruct *curves;

int main(int argc, char *argv[])
{
  Widget toplevel, draw;
  int i;
  windowInfo *wInfo;
  CurveStruct *curve;
  hsGeneral *item;
  wInfo = (windowInfo *)XtMalloc(sizeof(windowInfo));
  nCurves = readData(&curves);
  toplevel = XtInitialize(argv[0], "CurvesTest", NULL, 0, &argc, argv);
  draw = XtVaCreateManagedWidget("test", curvesWidgetClass, toplevel, NULL); 
  for (i = 0; i < nCurves; i++) 
    wInfo->ntVars[i] = 1;
  wInfo->ntVars[nCurves] = -1;
  wInfo->shell = toplevel;
  wInfo->widget = draw;
  if (argc == 2) {
    if (!strcmp(argv[1], "XY")) {
      CreateXYPlotMenu(wInfo);
      redisplayXY(wInfo, item, True);
    }
    else {
      CreateTSPlotMenu(wInfo);
      redisplayTS(wInfo, item, True);
    }
  }
  else {
      CreateTSPlotMenu(wInfo);
      redisplayTS(wInfo, item, True);
  }
  XtRealizeWidget(toplevel);
  XtMainLoop();

  XtFree((char *) curves);
  exit(0);
}

static int readData(CurveStruct **curves) {

  FILE *file_ptr;
  int i, j, num_point, nCurves, len;
  float data;
  char variable_name[91], chr;
  CurveStruct *curve;

  if ((file_ptr = fopen("testCurves.dat", "r")) == NULL) {
    printf("\nNot found data file %s\n", "testCurves.dat");
    exit(-1);
  }

  fscanf(file_ptr, " %d", &nCurves);
  while ((chr = getc(file_ptr)) != '\n');

  *curves = (CurveStruct *)XtMalloc(sizeof(CurveStruct) * nCurves);
  curve = *curves;

  for (i = 0; i < nCurves; i++) {
    len = 0;
    while ((variable_name[len] = fgetc(file_ptr)) != '\n')
      len++;
    variable_name[len] = '\0'; 
    curve->variable_name = (char *) XtMalloc(sizeof(char)*len +1);
    strcpy(curve->variable_name, variable_name);

    fscanf(file_ptr," %d, %d", &num_point, &curve->options);
    curve->num_point = num_point;

    if (num_point == 0)
      curve->points = NULL;
    else {
      curve->points = (float *)XtMalloc(sizeof(float)*num_point);
      for (j = 0; j < num_point; j++) {
        fscanf(file_ptr, " %f", &data);
        curve->points[j] = data;
      }
    }

    while ((chr = getc(file_ptr)) != '\n')
      if (chr == EOF)
        break;

    curve++;
  } /* for */
  return nCurves;
}

void CreateTSPlotMenu(windowInfo *wInfo)
{
      
    createPlotMenu(wInfo);
    addMenuToggle(wInfo->menu, "stats", "Show Statistics", 't', "Ctrl<Key>t",
    		  "Ctrl T", (XtCallbackProc)scatStatsCB, wInfo, False);
    addMenuSeparator(wInfo->menu, "s1");
    addMenuItem(wInfo->menu, "axis", "Axis Settings...", 'A', "Ctrl<Key>a",
    		"Ctrl A",  (XtCallbackProc)scatAxisCB, wInfo);
    addMenuItem(wInfo->menu, "setMark&LineStyle", "Set Mark & Line Style", 
                'M', "Ctrl<Key>m",
    		"Ctrl M",  (XtCallbackProc)curvesSetMarkLineStyleCB, wInfo);
    addMenuItem(wInfo->menu, "scaleVars", "Scale Variables", 'S', "Ctrl<Key>s",
    		"Ctrl S",  (XtCallbackProc)curvesScaleVariablesCB, wInfo);
    addMenuToggle(wInfo->menu, "showLegend", "Show Legend", 'g', "Ctrl<Key>G",
    		  "Ctrl G", (XtCallbackProc)curvesShowLegendCB, wInfo, 
    		  showLegend(wInfo, True));
    addMenuItem(wInfo->menu, "zoomIn", "Zoom In", 'I', "Ctrl<Key>z",
    		"Ctrl Z",  (XtCallbackProc)curvesZoomInCB, wInfo);
    addMenuItem(wInfo->menu, "zoomOut", "Zoom Out", 'O', "Shift Ctrl<Key>z",
    		"Shift Ctrl Z", (XtCallbackProc)curvesZoomOutCB, wInfo);
    addMenuItem(wInfo->menu, "resetZoom", "Zoom Reset", 'R', "Ctrl<Key>r",
    		"Ctrl R", (XtCallbackProc)curvesResetZoomCB, wInfo);
    addMenuSeparator(wInfo->menu, "s2");
    addMenuToggle(wInfo->menu, "xLogScaling", "xLogScaling", 'x', "Ctrl<Key>x",
    		  "Ctrl X", (XtCallbackProc)toggleXLogCB, wInfo, False);
    addMenuToggle(wInfo->menu, "yLogScaling", "yLogScaling", 'y', "Ctrl<Key>y",
    		  "Ctrl Y", (XtCallbackProc)toggleYLogCB, wInfo, False);
    addMenuSeparator(wInfo->menu, "s3");
    addMenuItem(wInfo->menu, "print", "Print...", 'P', "Ctrl<Key>p",
    		"Ctrl P",  (XtCallbackProc)curvesPrintCB, wInfo);
    addMenuSeparator(wInfo->menu, "s4");
    addMenuItem(wInfo->menu, "close", "Close", 'C', "Ctrl<Key>w", "Ctrl W", 
    		(XtCallbackProc)closeCB, wInfo);
}

void CreateXYPlotMenu(windowInfo *wInfo)
{
    createPlotMenu(wInfo);
    addMenuToggle(wInfo->menu, "stats", "Show Statistics", 't', "Ctrl<Key>t",
    		  "Ctrl T", (XtCallbackProc)scatStatsCB, wInfo, False);
    addMenuSeparator(wInfo->menu, "s1");
    addMenuItem(wInfo->menu, "axis", "Axis Settings...", 'A', "Ctrl<Key>a",
    		"Ctrl A",  (XtCallbackProc)scatAxisCB, wInfo);
    addMenuItem(wInfo->menu, "setMark&LineStyle", "Set Mark & Line Style", 
                'M', "Ctrl<Key>m",
    		"Ctrl M",  (XtCallbackProc)curvesSetMarkLineStyleCB, wInfo);
    addMenuItem(wInfo->menu, "scaleVars", "Scale Variables", 'S', "Ctrl<Key>s",
    		"Ctrl S",  (XtCallbackProc)curvesScaleVariablesCB, wInfo);
    addMenuToggle(wInfo->menu, "showLegend", "Show Legend", 'g', "Ctrl<Key>G",
    		  "Ctrl G", (XtCallbackProc)curvesShowLegendCB, wInfo, 
    		  showLegend(wInfo, False));
    addMenuItem(wInfo->menu, "zoomIn", "Zoom In", 'I', "Ctrl<Key>z",
    		"Ctrl Z",  (XtCallbackProc)curvesZoomInCB, wInfo);
    addMenuItem(wInfo->menu, "zoomOut", "Zoom Out", 'O', "Shift Ctrl<Key>z",
    		"Shift Ctrl Z", (XtCallbackProc)curvesZoomOutCB, wInfo);
    addMenuItem(wInfo->menu, "resetZoom", "Zoom Reset", 'R', "Ctrl<Key>r",
    		"Ctrl R", (XtCallbackProc)curvesResetZoomCB, wInfo);
    addMenuToggle(wInfo->menu, "xLogScaling", "xLogScaling", 'x', "Ctrl<Key>x",
    		  "Ctrl X", (XtCallbackProc)toggleXLogCB, wInfo, False);
    addMenuToggle(wInfo->menu, "yLogScaling", "yLogScaling", 'y', "Ctrl<Key>y",
    		  "Ctrl Y", (XtCallbackProc)toggleYLogCB, wInfo, False);
    addMenuSeparator(wInfo->menu, "s2");
    addMenuItem(wInfo->menu, "print", "Print...", 'P', "Ctrl<Key>p",
    		"Ctrl P",  (XtCallbackProc)curvesPrintCB, wInfo);
    addMenuSeparator(wInfo->menu, "s4");
    addMenuItem(wInfo->menu, "close", "Close", 'C', "Ctrl<Key>w", "Ctrl W", 
    		(XtCallbackProc)closeCB, wInfo);
}

static void scatStatsCB(Widget w, windowInfo *wInfo, caddr_t callData)
{
    printf("Show Stats\n");
}                  
void CreateTSPlotWindow(Display *display, hsGeneral *item, int *ntVars, char *title)
{
  windowInfo *wInfo;

    /* create the window */

    /* add the appropriate menu items */
    CreateTSPlotMenu(wInfo);

    /* initialize the data in the histogram */
    redisplayTS(wInfo, item, True);

    /* Showtime */
    XtRealizeWidget(wInfo->shell);
}


void CreateXYPlotWindow(Display *display, hsGeneral *item, int *ntVars, char *title)
{
  windowInfo *wInfo;

    /* create the window */

    /* add the appropriate menu items */
    CreateXYPlotMenu(wInfo);

    /* initialize the data in the histogram */
    redisplayXY(wInfo, item, True);

    /* Showtime */
    XtRealizeWidget(wInfo->shell);
}

static void redisplayTSandXY(windowInfo *window, Boolean isTimeSeries)
{
   
   CurvesSetContents(window->widget, curves, nCurves, CURVES_RESCALE,
                     isTimeSeries);
}

static void redisplayTS(windowInfo *window, hsGeneral *item, int reInit)
{
  redisplayTSandXY(window, True);
}

static void redisplayXY(windowInfo *window, hsGeneral *item, int reInit)
{
   redisplayTSandXY(window, False);
}

static void createPlotMenu(windowInfo *wInfo)
{
    wInfo->menu = XmCreatePopupMenu(wInfo->widget, "plotMenu", NULL, 0);
    XtAddEventHandler(wInfo->widget, ButtonPressMask, False,
    		      (XtEventHandler)popupCB, (XtPointer)wInfo->menu);
    AddMotifCloseCallback(wInfo->shell, (XtCallbackProc)closeCB, wInfo);
}

static void popupCB(Widget w, Widget menu, XButtonEvent *event)
{
    if (event->button != Button3)
    	return;
    	
    XmMenuPosition(menu, event);
    XtManageChild(menu);
}

static Widget addMenuItem(Widget parent, char *name, char *label,
			  char mnemonic, char *acc, char *accText,
			  XtCallbackProc callback, void *cbArg)
{
    Widget button;
    XmString st1, st2;
    
    button = XtVaCreateManagedWidget(name, xmPushButtonWidgetClass, parent, 
    	XmNlabelString, st1=XmStringCreateSimple(label),
    	XmNmnemonic, mnemonic,
    	XmNacceleratorText, st2=XmStringCreateSimple(accText),
    	XmNaccelerator, acc, NULL);
    XtAddCallback(button, XmNactivateCallback, callback, cbArg);
    XmStringFree(st1);
    XmStringFree(st2);
    return button;
}

static Widget addMenuToggle(Widget parent, char *name, char *label,
		 	    char mnemonic, char *acc, char *accText,
		  	    XtCallbackProc callback, void *cbArg, int set)
{
    Widget button;
    XmString st1, st2;
    
    button = XtVaCreateManagedWidget(name, xmToggleButtonWidgetClass, parent, 
    	XmNlabelString, st1=XmStringCreateSimple(label),
    	XmNmnemonic, mnemonic,
    	XmNacceleratorText, st2=XmStringCreateSimple(accText),
    	XmNaccelerator, acc,
    	XmNset, set, NULL);
    XtAddCallback(button, XmNvalueChangedCallback, callback, cbArg);
    XmStringFree(st1);
    XmStringFree(st2);
    return button;
}

static Widget addMenuSeparator(Widget parent, char *name)
{
    Widget button;
    
    button = XmCreateSeparator(parent, name, NULL, 0);
    XtManageChild(button);
    return button;
}


static void scatAxisCB(Widget w, windowInfo *wInfo, caddr_t callData)
{
    printf("Axis Settings\n");
}
static void closeCB(Widget w, windowInfo *wInfo, caddr_t callData)
{
  
  XtFree((char *) wInfo);
  XtCloseDisplay(XtDisplay(w));
  exit(0);
}

static void toggleXLogCB(Widget w, windowInfo *wInfo, caddr_t callData)
{
    XtVaSetValues(wInfo->widget, XmNxLogScaling, XmToggleButtonGetState(w), 0);
}
static void toggleYLogCB(Widget w, windowInfo *wInfo, caddr_t callData)
{
    XtVaSetValues(wInfo->widget, XmNyLogScaling, XmToggleButtonGetState(w), 0);
}


/*
** callbacks for Curves Plot Widget
*/
static void curvesZoomInCB(Widget w, windowInfo *wInfo, caddr_t callData)
{
    CurvesZoomIn(wInfo->widget);
}
static void curvesZoomOutCB(Widget w, windowInfo *wInfo, caddr_t callData)
{
    CurvesZoomOut(wInfo->widget);
}
static void curvesResetZoomCB(Widget w, windowInfo *wInfo, caddr_t callData) 
{
    CurvesResetZoom(wInfo->widget);
}
static void curvesSetMarkLineStyleCB(Widget w, windowInfo *wInfo, caddr_t callData)
{

  CurvesSetMarkLineStyle(wInfo->widget);
}
static void curvesShowLegendCB(Widget w, windowInfo *wInfo, caddr_t callData)
{
  CurvesShowLegend(wInfo->widget, XmToggleButtonGetState(w));
}
static void curvesScaleVariablesCB(Widget w, windowInfo *wInfo, caddr_t callData)
{
  CurvesScaleVariables(wInfo->widget);
}
static Boolean showLegend(windowInfo *wInfo, Boolean isTimeSeries) 
{
  int i, num_var = 0;
  
  for (i = 0; i < MAX_DISP_VARS; i++)
    if (wInfo->ntVars[i] != -1)
      num_var++;
  
  if (isTimeSeries && num_var == 1)
    return False;
  if (!isTimeSeries && num_var == 2)
    return False;
    
  return True;
}
static void showSlidersCB(Widget w, windowInfo *wInfo, caddr_t callData)
{
    printf("Show Sliders\n");
}

/*
** Callbacks for 2D Scatter Plot Widget
*/
static void scatResetCB(Widget w, windowInfo *wInfo, caddr_t callData)
{
    printf("Reset Zoom\n");
}
static void curvesPrintCB(Widget w, windowInfo *wInfo, caddr_t callData)
{
    char tmpFileName[L_tmpnam];    /* L_tmpnam defined in stdio.h */
    char *windowTitle;

    tmpnam(tmpFileName);
    CurvesPrintContents(wInfo->widget, tmpFileName);
    XtVaGetValues(wInfo->shell, XmNtitle, &windowTitle, 0);
#ifdef VMS
    PrintFile(wInfo->widget, tmpFileName, windowTitle, True);
#else
    PrintFile(wInfo->widget, tmpFileName, windowTitle);
#endif /*VMS*/
    remove(tmpFileName);
}
                                                                                   
