/********************************************************************
*                                                                   *
* testXY.c - XY Plot Widget Example Program                         *
*                                                                   *
* This program is an example program that demonstrates the XY       *
* widget capabilities                                               *
*                                                                   *
*                                                                   *
* July 15, 1994 by George Dimas, Mark Edel                          *
*                                                                   *
********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#ifdef notdef
#include <string.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#endif
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/ToggleB.h>
#include <Xm/PushB.h>
#include <Xm/MessageB.h>
#include <Xm/DialogS.h>
#include <Xm/Text.h>
#include <Xm/LabelG.h>
#include "XY.h"
#include "XYDialogs.h"
#include "../util/printUtils.h"
#include "../util/DialogF.h"

#define NUM_POINTS  50
#define NUM_CURVES  3
#define NUM_HISTOGRAMS 3
#define NUM_BINS 30

Widget XYPlot, TimeSeriesToggle, ErrorBarsToggle;
Widget RangeText[4];

static void createButton(Widget parent, char *name, char *label,
	void callbackProc(Widget w, caddr_t client_d, caddr_t call_data), 
	XtPointer cbArg);
static Widget createToggleButton(Widget parent, char *name, char *label,
	Boolean initialState, void callbackProc(Widget w, caddr_t client_data,
	caddr_t call_data), XtPointer cbArg);
static void logXCB(Widget w, caddr_t client_data, caddr_t call_data);
static void logYCB(Widget w, caddr_t client_data, caddr_t call_data);
static void doubleBufferCB(Widget w, caddr_t client_d, caddr_t call_d);
static void setRangeCB(Widget w, caddr_t client_data, caddr_t call_data);
static void rangeOkCB(Widget w, Widget x, caddr_t call_data);
static void rangeCancelCB(Widget w, Widget x, caddr_t call_data);
static void createWarningMessage(Widget parent, char *text);
static void getRangeCB(Widget w, caddr_t client_data, caddr_t call_data);
static void scaleVariablesCB(Widget w, caddr_t client_data, caddr_t call_data);
static void setMarkLineStyleCB(Widget w, caddr_t client_d, caddr_t call_data);
static void setHistogramStyleCB(Widget w, caddr_t client_d, caddr_t call_data);
static void styleApplyCB(XYCurve *curves, int nCurves, void *arg);
static void styleOKCB(XYCurve *curves, int nCurves, void *arg);
static void showLegendCB(Widget w, caddr_t client_d, caddr_t call_d);
static void stringsCB(Widget w, XtPointer client_d, caddr_t call_d);
static void errorBarsCB(Widget w, XtPointer client_d, caddr_t call_d);
static void createCurves(Boolean init, Boolean showErrorBars);
static void createHistograms(Boolean init, Boolean showErrorBars);
static void zoomCB(Widget w, XtPointer data, caddr_t call_data);
static void printCB(Widget w, caddr_t client_d, caddr_t call_d);
static void btn2CB(Widget w, XtPointer data, XYCallbackStruct *callData);
static void btn3CB(Widget w, XtPointer data, XYCallbackStruct *callData);
static void exitCB(Widget w, caddr_t client_data, caddr_t call_data);

static XYPoint Data[NUM_CURVES][NUM_POINTS];
static XYErrorBar HBars[NUM_CURVES][NUM_POINTS];
static XYErrorBar VBars[NUM_CURVES][NUM_POINTS];
static float BinData[NUM_HISTOGRAMS][NUM_BINS];
static XYErrorBar HistErrBars[NUM_HISTOGRAMS][NUM_BINS];
static XYCurve Curves[] = {
    {NULL, XY_SQUARE_MARK, XY_SMALL, XY_2_DOT_DASH,
    	    0, 1, NUM_POINTS, Data[0], NULL, NULL},
    {NULL, XY_SQUARE_MARK, XY_LARGE,  XY_FINE_DASH,
    	    2, 3, NUM_POINTS, Data[1], NULL, NULL},
    {NULL, XY_X_MARK, XY_SMALL,  XY_MED_FINE_DASH,
    	    4, 5, NUM_POINTS, Data[2], NULL, NULL},
    {NULL, XY_CIRCLE_MARK,  XY_MEDIUM, XY_DASH,
    	    6, 7, NUM_POINTS, Data[3], NULL, NULL},
    {NULL, XY_SOLID_SQUARE_MARK,  XY_SMALL, XY_LONG_DASH,
    	    8, 9, NUM_POINTS, Data[4], NULL, NULL},
    {NULL, XY_TRIANGLE_MARK,  XY_SMALL, XY_X_LONG_DASH,
    	    10, 11, NUM_POINTS, Data[5], NULL, NULL},
    {NULL, XY_STAR_MARK,  XY_SMALL, XY_1_DOT_DASH,
    	    12, 13, NUM_POINTS, Data[6], NULL, NULL},
    {NULL, XY_X_MARK,  XY_TINY, XY_2_DOT_DASH,
    	    14, 15, NUM_POINTS, Data[7], NULL, NULL},
    {NULL, XY_NO_MARK, XY_SMALL,  XY_2_DOT_DASH,
    	    16, 16, NUM_POINTS, Data[8], NULL, NULL},
    {NULL, XY_SQUARE_MARK, XY_LARGE, XY_3_DOT_DASH,
    	    18, 19, NUM_POINTS, Data[9], NULL, NULL},
    {NULL, XY_THICK_CIRCLE_MARK, XY_LARGE, XY_4_DOT_DASH,
    	    20, 21, NUM_POINTS, Data[10], NULL, NULL},
    {NULL, XY_THICK_CIRCLE_MARK, XY_MEDIUM, XY_THICK_LINE,
    	    22, 23, NUM_POINTS, Data[11], NULL, NULL},
    {NULL, XY_THICK_CIRCLE_MARK, XY_SMALL, XY_X_THICK_LINE,
    	    24, 25, NUM_POINTS, Data[12], NULL, NULL},
    {NULL, XY_THICK_CIRCLE_MARK, XY_TINY, XY_1_DOT_DASH,
    	    26, 27, NUM_POINTS, Data[13], NULL, NULL},
    {NULL, XY_THICK_SQUARE_MARK, XY_LARGE, XY_NO_LINE,
    	    28, 29, NUM_POINTS, Data[14], NULL, NULL},
};

static XYHistogram Histograms[] = {
    {NULL, XY_NO_LINE, XY_SOLID_FILL, 3, 4, 0., 100., NUM_BINS, BinData[2], NULL, 0., NULL},
    {NULL, XY_DASH, XY_FINE_X_FILL, 0, 1, 0., 100., NUM_BINS, BinData[0], NULL, 0., NULL},
    {NULL, XY_PLAIN_LINE, XY_NO_FILL, 2, 3, 0., 100., NUM_BINS, BinData[1], NULL, 0., NULL},
};

main(int argc, char *argv[])
{
    XtAppContext appContext;
    Widget toplevel, form, checkBox, rowColumn;
    XmString s1, s2;
    
    /* Initialize X and Xt */
    toplevel = XtAppInitialize(&appContext, "XYTest", NULL,
	    0, &argc, argv, NULL, NULL, 0);
    
    /* Create a form Widget as the container for a row column,
       check box and XY widget. */
    form = XtCreateManagedWidget("form",
    	xmFormWidgetClass, toplevel, NULL, 0);
    
    /* Create an XY widget. */
    XYPlot = XtVaCreateManagedWidget("XYPlot",
    	    xyWidgetClass, form, 
    	    XmNxAxisLabel, s1=XmStringCreateSimple("X axis label"),
    	    XmNyAxisLabel, s2=XmStringCreateSimple("Y axis label"),
    	    /* XmNmarginPercent, 0, */
    	    XmNtopAttachment, XmATTACH_FORM,
    	    XmNleftAttachment, XmATTACH_POSITION,
    	    XmNleftPosition, 21,
    	    XmNrightAttachment, XmATTACH_FORM,
    	    XmNbottomAttachment, XmATTACH_FORM,
    	    NULL);
    XtAddCallback(XYPlot, XmNbtn2Callback, (XtCallbackProc)btn2CB, NULL);
    XtAddCallback(XYPlot, XmNbtn3Callback, (XtCallbackProc)btn3CB, NULL);
    XmStringFree(s1);
    XmStringFree(s2);

    /* Draw the example curves and histograms */
    createCurves(TRUE, TRUE);
    createHistograms(TRUE, TRUE);
    	
    /* Create a row column to hold toggle buttons. */
    checkBox = XtVaCreateManagedWidget("toggleButtons",
    	    xmRowColumnWidgetClass, form,
    	    XmNtopAttachment, XmATTACH_FORM,
    	    XmNleftAttachment, XmATTACH_FORM,
    	    XmNrightAttachment, XmATTACH_POSITION,
    	    XmNrightPosition, 20,
    	    NULL);
    
    /* Create the toggle buttons */
    createToggleButton(checkBox, "logX", "Log X", FALSE, logXCB, NULL);
    createToggleButton(checkBox, "logY", "Log Y", FALSE, logYCB, NULL);
    createToggleButton(checkBox, "showLegend",
    	    "Show Legend", TRUE, showLegendCB, NULL);
    TimeSeriesToggle = createToggleButton(checkBox, "addStrings",
    	    "Add Strings", FALSE, stringsCB, NULL);
    ErrorBarsToggle = createToggleButton(checkBox, "showErrorBars",
    	    "Show Error Bars", TRUE, errorBarsCB, NULL);
    createToggleButton(checkBox, "doubleBuffer",
    	    "Double Buffer", FALSE, doubleBufferCB, NULL);
    
    /* Create a row column to hold the push buttons. */
    rowColumn = XtVaCreateManagedWidget("rowColumn",
    	    xmRowColumnWidgetClass, form,
    	    XmNtopAttachment, XmATTACH_WIDGET,
    	    XmNtopWidget, checkBox,
    	    XmNtopOffset, 10,
    	    XmNleftAttachment, XmATTACH_FORM,
    	    XmNrightAttachment, XmATTACH_POSITION,
    	    XmNrightPosition, 20,
    	    XmNbottomAttachment, XmATTACH_FORM,
    	    NULL);
    
    /* Create the push buttons. */
    createButton(rowColumn, "setMarkLineStyle", "Set Mark & Line Style...",
	    setMarkLineStyleCB, NULL);
    createButton(rowColumn, "setHistogramStyle", "Set Histogram Style...",
	    setHistogramStyleCB, NULL);
    createButton(rowColumn, "scaleVariables", "Scale Variables...",
	    scaleVariablesCB, NULL);
    createButton(rowColumn, "setRange", "Set Visible Range...",
	    setRangeCB, NULL);
    createButton(rowColumn, "getRange", "Get Visible Range...",
	    getRangeCB, NULL);
    createButton(rowColumn, "zoomIn", "Zoom In", zoomCB, (XtPointer)1);
    createButton(rowColumn, "zoomOut", "Zoom Out", zoomCB, (XtPointer)2);
    createButton(rowColumn, "resetZoom", "Reset Zoom",
    	    zoomCB, (XtPointer)3);
    createButton(rowColumn, "printPlot", "Print Plot...",
	    printCB, NULL);
    createButton(rowColumn, "exitProgram", "Exit Program",
	    exitCB, NULL);
    
    /* Realize widgets and call XtMainLoop to continuously process events. */
    XtRealizeWidget(toplevel);
    XtAppMainLoop(appContext);
}

static void createButton(Widget parent, char *name, char *label,
	void callbackProc(Widget w, caddr_t client_d, caddr_t call_data), 
	XtPointer cbArg)
{
    XmString s1;
    Widget button;
    	   	  	
    button = XtVaCreateManagedWidget(name, xmPushButtonWidgetClass, parent,
    	    XmNlabelString, s1=XmStringCreateSimple(label), NULL);
    XtAddCallback(button, XmNactivateCallback,
    	    (XtCallbackProc)callbackProc, cbArg); 
    XmStringFree(s1);
}

static Widget createToggleButton(Widget parent, char *name, char *label,
	Boolean initialState, void callbackProc(Widget w, caddr_t client_data,
	caddr_t call_data), XtPointer cbArg)
{
    XmString s1;
    Widget button;
    	   	  	
    button = XtVaCreateManagedWidget(name, xmToggleButtonWidgetClass, parent,
    	    XmNlabelString, s1=XmStringCreateSimple(label),
    	    XmNset, initialState, NULL);
    XtAddCallback(button, XmNvalueChangedCallback,
    	    (XtCallbackProc)callbackProc, cbArg); 
    XmStringFree(s1);
    return button;
}

static void logXCB(Widget w, caddr_t client_data, caddr_t call_data)
{ 
    /* This routine is activated every time toggle button Log X has
       been pressed. It changes the X axis from linear to log scaling
       and vice versa. */
    
    XtVaSetValues(XYPlot, XmNxLogScaling, XmToggleButtonGetState(w), NULL);
}

static void logYCB(Widget w, caddr_t client_data, caddr_t call_data)
{
    
    /* This routine is activated every time toggle button Log Y has
       been pressed. It changes the Y axis from linear to log scaling
       and vice versa. */
    
    XtVaSetValues(XYPlot, XmNyLogScaling, XmToggleButtonGetState(w), NULL);
}

static void doubleBufferCB(Widget w, caddr_t client_d, caddr_t call_d)
{  
    /* This routine is activated every time toggle button Double Buffer 
       has been pressed. It turns on and off graphics buffering */
    
    XtVaSetValues(XYPlot, XmNdoubleBuffer, XmToggleButtonGetState(w),
    	    NULL);
}

static void setRangeCB(Widget w, caddr_t client_data, caddr_t call_data)
{
    /* This function will create and pop-up a form dialog, in which a user
       can enter the maximum and minimum values for the new range.
       Also, included is a Cancel and an Ok push button. The callback for
       the Ok pushbutton actually sets the visible range */ 
       
    Widget form, labelMinX, labelMinY, labelMaxX, labelMaxY, ok, cancel;
    XmString xmstr; 
    
    /* Create a form widget to contain the text & label widgets. */
    form = XmCreateFormDialog(w, "form", NULL, 0);
    XtVaSetValues(form, 
    	    XmNdialogTitle, xmstr=XmStringCreateSimple("Set Range"),
    	    XmNautoUnmanage, FALSE, 
    	    NULL);
    XmStringFree(xmstr); 
    

    /* Create 4 label & 4 text widgets. */
    labelMinX = XtVaCreateManagedWidget("labelMinX", 
    	    xmLabelGadgetClass, form,
    	    XmNlabelString, xmstr=XmStringCreateSimple("min X"),
    	    XmNtopAttachment, XmATTACH_FORM,
    	    XmNtopOffset, 10,
    	    XmNleftAttachment, XmATTACH_FORM,
    	    XmNleftOffset, 10,
    	    NULL);
    XmStringFree(xmstr); 
    
    RangeText[0] = XtVaCreateManagedWidget("textMinX", 
    	    xmTextWidgetClass, form,
    	    XmNtopAttachment, XmATTACH_FORM,
    	    XmNtopOffset, 5,
    	    XmNleftAttachment, XmATTACH_WIDGET,
    	    XmNleftWidget, labelMinX,
    	    NULL); 
    	
    labelMinY = XtVaCreateManagedWidget("labelMinY", 
    	    xmLabelGadgetClass, form,
    	    XmNlabelString, xmstr=XmStringCreateSimple("min Y"),
    	    XmNtopAttachment, XmATTACH_FORM,
    	    XmNtopOffset, 10,
    	    XmNleftAttachment, XmATTACH_WIDGET,
    	    XmNleftWidget, RangeText[0],
    	    XmNleftOffset, 10,
    	    NULL);
    XmStringFree(xmstr);
    	
    RangeText[1] = XtVaCreateManagedWidget("textMinY", 
    	    xmTextWidgetClass, form,
    	    XmNtopAttachment, XmATTACH_FORM,
    	    XmNtopOffset, 5,
    	    XmNleftAttachment, XmATTACH_WIDGET,
    	    XmNleftWidget, labelMinY,
    	    NULL);
    	
    labelMaxX = XtVaCreateManagedWidget("labelMaxX", 
    	    xmLabelGadgetClass, form,
    	    XmNlabelString, xmstr=XmStringCreateSimple("max X"),
    	    XmNtopAttachment, XmATTACH_WIDGET,
    	    XmNtopWidget, labelMinX,
    	    XmNtopOffset, 20,
    	    XmNleftAttachment, XmATTACH_FORM,
    	    XmNleftOffset, 10,
    	    NULL);
    XmStringFree(xmstr);
    
    RangeText[2] = XtVaCreateManagedWidget("textMaxX", 
    	    xmTextWidgetClass, form,
    	    XmNtopAttachment, XmATTACH_WIDGET,
    	    XmNtopWidget, RangeText[0],
    	    XmNtopOffset, 5,
    	    XmNleftAttachment, XmATTACH_WIDGET,
    	    XmNleftWidget, labelMaxX,
    	    NULL); 
    	
    labelMaxY = XtVaCreateManagedWidget("labelMaxY", 
            xmLabelGadgetClass, form,
            XmNlabelString, xmstr=XmStringCreateSimple("max Y"),
            XmNtopAttachment, XmATTACH_WIDGET,
    	    XmNtopWidget, labelMinY,
    	    XmNtopOffset, 20,
    	    XmNleftAttachment, XmATTACH_WIDGET,
    	    XmNleftWidget, RangeText[2],
    	    XmNleftOffset, 10,
    	    NULL);
    XmStringFree(xmstr); 
    
    RangeText[3] = XtVaCreateManagedWidget("textMaxY", 
    	    xmTextWidgetClass, form,
    	    XmNtopAttachment, XmATTACH_WIDGET,
    	    XmNtopWidget, RangeText[1],
    	    XmNtopOffset, 5, 
    	    XmNleftAttachment, XmATTACH_WIDGET,
    	    XmNleftWidget, labelMaxY,
    	    NULL);	

    	
    /* Create OK push button widget and callback */
    ok = XtVaCreateManagedWidget("ok", 
    	    xmPushButtonWidgetClass, form,
    	    XmNlabelString, xmstr=XmStringCreateSimple(" OK "),
    	    XmNtopAttachment, XmATTACH_WIDGET,
    	    XmNtopWidget, RangeText[2],
    	    XmNtopOffset, 20,
    	    XmNleftAttachment, XmATTACH_FORM,
    	    XmNleftOffset, 20,  
    	    NULL);
    XtAddCallback(ok, XmNactivateCallback, 
    	    (XtCallbackProc) rangeOkCB, (void *) form);
    XmStringFree(xmstr);
    
    /* Create Cancel push button widget and callback */ 
    cancel = XtVaCreateManagedWidget("cancel", 
    	    xmPushButtonWidgetClass, form,
    	    XmNlabelString, xmstr=XmStringCreateSimple("Cancel"),
    	    XmNtopAttachment, XmATTACH_WIDGET,
    	    XmNtopWidget, RangeText[2],
    	    XmNtopOffset, 20,
    	    XmNleftAttachment, XmATTACH_WIDGET,
    	    XmNleftWidget, ok,
    	    XmNleftOffset, 5, 
    	    NULL);
    XtAddCallback(cancel, XmNactivateCallback, 
    	    (XtCallbackProc) rangeCancelCB, (void *) form);
    XmStringFree(xmstr);

    XtManageChild(form);
}

static void rangeOkCB(Widget w, Widget x, caddr_t call_data)
{
    /* This function will be called after the user has entered the 
       maximum and minimum values from the setRangeCB. This
       function first calls XYGetVisibleRange to get the current 
       range. If the user leaves one field blank, the field will not  
       be affected. If the user changed some fields, this function 
       converts the text from char to double and passes the value 
       to the function XYSetVisibleRange; */
    
    char *t;   
    double tempDouble, minXLim, maxXLim, minYLim, maxYLim;
    char *str, *ptr;
    
    /* Find the initial visible range to supply default values */
    XYGetVisibleRange(XYPlot, &minXLim, &minYLim, &maxXLim,
    	    &maxYLim);
    
    /* Get text from the 4 widgets and convert string from char to double. */
    str = XmTextGetString(RangeText[0]);
    tempDouble = strtod(str, &ptr);
    if (ptr != str)
        minXLim = tempDouble;
    XtFree(str);
    
    str = XmTextGetString(RangeText[1]);
    tempDouble = strtod(str, &ptr); 
    if (ptr != str)
        minYLim = tempDouble;
    XtFree(str);
    
    str = XmTextGetString(RangeText[2]);
    tempDouble = strtod(str, &ptr);
    if (ptr != str)
        maxXLim = tempDouble; 
    XtFree(str);
    
    str = XmTextGetString(RangeText[3]);
    tempDouble = strtod(str, &ptr);
    if (ptr != str)
        maxYLim = tempDouble;
    XtFree(str);
   
    if (minXLim >= maxXLim || minYLim >= maxYLim) 
	createWarningMessage(x, "Can't set visible range.\nCheck values");
    else { 
	XtDestroyWidget(XtParent(x));
        XYSetVisibleRange(XYPlot, minXLim, minYLim, maxXLim,
        	maxYLim); 
    }
}

static void rangeCancelCB(Widget w, Widget x, caddr_t call_data)
{
    XtDestroyWidget(XtParent(x));
}

static void createWarningMessage(Widget parent, char *text)
{
    /* This function creates a warning dialog that displays 
       the text from parameter "text".  The dialogs created here
       are never destroyed, so don't use this in a real program */
        
    Widget message;
    Arg wargs[5];
    XmString xmstr, xmWarning;
    int n;
    
    /* Create a warning dialog widget to display a message. */
    n = 0;
    xmstr = XmStringCreateLtoR(text, XmSTRING_DEFAULT_CHARSET);
    xmWarning = XmStringCreateSimple("Warning");
    XtSetArg(wargs[n], XmNmessageString, xmstr);    n++;
    XtSetArg(wargs[n], XmNdialogTitle, xmWarning);   n++; 
    message = XmCreateWarningDialog(parent, "message", wargs, n);
    XmStringFree(xmstr);
    XmStringFree(xmWarning);
    
    /* Unmanage the Cancel and Help widgets, since we won't need them. */ 
    XtUnmanageChild(XmMessageBoxGetChild (message, XmDIALOG_CANCEL_BUTTON));
    XtUnmanageChild(XmMessageBoxGetChild (message, XmDIALOG_HELP_BUTTON));
    
    /* Manage and pop-up the dialog */
    XtManageChild(message);
}    

static void getRangeCB(Widget w, caddr_t client_data, caddr_t call_data)
{
   /* This function creates a message dialog that displays the coordinates
       that are returned from XYGetVisibleRange. */
       
    double minXLim, maxXLim, minYLim, maxYLim;
    Widget infoBox;
    XmString xmstr1, xmstr2;
    char message[100];
    Arg wargs[5];
    int n;
   
    XYGetVisibleRange(XYPlot, &minXLim, &minYLim, &maxXLim,
    	    &maxYLim);           
   
    /* Create the string for display. */
    sprintf(message, "min X = %g, min Y = %g, max X = %g, max Y = %g",
            minXLim, minYLim, maxXLim, maxYLim);
    
    /* Create a messaage dialog to display coordinates. */
    n = 0;
    xmstr1 = XmStringCreateSimple("Visible Range");
    xmstr2 = XmStringCreateLtoR(message, XmSTRING_DEFAULT_CHARSET); 
    XtSetArg(wargs[n], XmNmessageString, xmstr2 ); n++;
    XtSetArg(wargs[n], XmNdialogTitle, xmstr1); n++; 
    infoBox = XmCreateMessageDialog(w, "infoBox", wargs, n);
    XmStringFree(xmstr1);  
    XmStringFree(xmstr2);   
      
    /* Unmanage the Cancel and Help widgets, since we won't need them. */ 
    XtUnmanageChild(XmMessageBoxGetChild (infoBox, XmDIALOG_CANCEL_BUTTON));
    XtUnmanageChild(XmMessageBoxGetChild (infoBox, XmDIALOG_HELP_BUTTON));
    
    XtManageChild(infoBox);
}

static void scaleVariablesCB(Widget w, caddr_t client_data, caddr_t call_data)
{
    /* This function calls XYScaleVariables, which allows the
       user to change the variable scaling. */
       
/*    CurvesScaleVariables(CurveWidget); */
}

static void setMarkLineStyleCB(Widget w, caddr_t client_d, caddr_t call_data)
{
    /* This function calls CurvesSetMarkLineStyle, which allows the
       user to mark the critical points (the point where the curve changes 
       direction) with different styles of markers and also change the
       style of the line in each curve. */
       
    /* XYCreateStylesDialog(XYPlot, Curves, NUM_CURVES, styleOKCB, (void *)1,
    	    styleApplyCB, (void *)2, NULL, NULL); */
    XYEditStyles(XYPlot, Curves, NUM_CURVES);
    XYUpdateStyles(XYPlot, Curves);
}

static void setHistogramStyleCB(Widget w, caddr_t client_d, caddr_t call_data)
{
    /* This function calls XYSetHistogramStyle, which allows the
       user to mark the critical points (the point where the curve changes 
       direction) with different styles of markers and also change the
       style of the line in each curve. */
       
    /* XYCreateStylesDialog(XYPlot, Curves, NUM_CURVES, styleOKCB, (void *)1,
    	    styleApplyCB, (void *)2, NULL, NULL); */
    XYEditHistogramStyles(XYPlot, Histograms, NUM_HISTOGRAMS);
    XYUpdateHistogramStyles(XYPlot, Histograms, True);
}

static void styleApplyCB(XYCurve *curves, int nCurves, void *arg)
{
    XYUpdateStyles(XYPlot, curves);
    printf("apply: arg = %d\n", arg);
}

static void styleOKCB(XYCurve *curves, int nCurves, void *arg)
{
    XYUpdateStyles(XYPlot, curves);
    printf("ok: arg = %d\n", arg);
}

static void showLegendCB(Widget w, caddr_t client_d, caddr_t call_d)
{
    /* This shows or hides the legend at the bottom of the plot */
        
    XtVaSetValues(XYPlot, XmNshowLegend, XmToggleButtonGetState(w), NULL);
}

static void stringsCB(Widget w, XtPointer client_d, caddr_t call_d)
{
    /* This function switches the widget back an forth between showing
       strings and not showing them */
       
    char str[25];
    int i;
    XYString *string, *strings;
    
    if (!XmToggleButtonGetState(w)) {
        XYSetStrings(XYPlot, NULL, 0);
        return;
    }
    
    strings = (XYString *)XtMalloc(sizeof(XYString) * NUM_POINTS);
    for (i=0, string=strings; i<NUM_POINTS; i++, string++) {
    	string->x = Curves[1].points[i].x;
    	string->y = Curves[1].points[i].y-2;
    	string->font = NULL;
    	string->color = i;
    	string->alignment = XY_CENTER;
    	sprintf(str, "This is point # %d", i+1);
    	string->string = XmStringCreateSimple(str);
    }
    XYSetStrings(XYPlot, strings, NUM_POINTS);
    for (i=0, string=strings; i<NUM_POINTS; i++, string++)
        XmStringFree(string->string);
    XtFree((char *)strings);
}

static void errorBarsCB(Widget w, XtPointer client_d, caddr_t call_d)
{
    /* This function switches the widget back an forth between showing
       or not showing error bars on curves and histograms. */
    int i, showErrorBars = XmToggleButtonGetState(ErrorBarsToggle);
    
    for (i=0; i<NUM_CURVES/2; i++) {
    	Curves[i].vertBars = showErrorBars ? VBars[i] : NULL;
    	Curves[i].horizBars = showErrorBars ? HBars[i] : NULL;
    }
    for (i=0; i<NUM_HISTOGRAMS/2; i++) {
    	Histograms[i].errorBars = showErrorBars ? HistErrBars[i] : NULL;
    }
    XYUpdateCurveData(XYPlot, Curves, XY_RESCALE_AT_MAX, False);
    XYUpdateHistogramData(XYPlot, Histograms, XY_RESCALE_AT_MAX, True);
}

static void createCurves(Boolean init, Boolean showErrorBars)
{
    Pixel black = BlackPixelOfScreen(XtScreen(XYPlot));
    int i, j;
    float xVal, yVal;
    char name[100];
#ifdef notdef
    static XYPoint data1[NUM_POINTS] = {{1, 1}, {2, 5}, {3, 12}, {4, 6},
    	    {5, 18}, {6, 4}, {7, 6}, {8, 9}, {9, 14}, {10, 13}};
    static XYPoint data2[NUM_POINTS] = {{1, 15}, {2, 4}, {3, 6}, {4, 8},
    	    {5, 12}, {6, 9}, {7, 4}, {8, 8}, {9, 25}, {10, 2}};
    static XYPoint data3[NUM_POINTS] = {{10, 15}, {9, 4}, {8, 6}, {7, 8},
    	    {6, 12}, {4, 9}, {3.8, 4}, {3.6, 8}, {3.4, 25}, {2, 2}};
    static XYErrorBar err2data[NUM_POINTS] = {{14, 16}, {3, 5}, {5, 7}, {7, 9},
    	   {11, 13}, {8, 10}, {3, 5}, {7, 9}, {24, 26}, {1, 3}};
    static XYErrorBar hErr2data[NUM_POINTS] = {{0, 2}, {1, 3}, {2, 4}, {3, 5},
    	   {4, 6}, {5, 7}, {6, 8}, {7, 9}, {8, 10}, {9, 11}};
#endif

    for (i=0; i<NUM_CURVES; i++) {
    	Curves[i].markerPixel = black;
    	Curves[i].linePixel = black;
    	sprintf(name, "Curve%d", i);
    	Curves[i].name = XmStringCreateSimple(name);
    }
    
    srand(1);
    for (i=0; i<NUM_CURVES; i++) {
    	for (j=0; j<NUM_POINTS; j++) {
    	    xVal = j - 10.;
    	    yVal = rand() / (RAND_MAX / 100) - 50;
    	    Curves[i].points[j].x = xVal;
    	    Curves[i].points[j].y = yVal;
    	    HBars[i][j].min = xVal - 1;
    	    HBars[i][j].max = xVal + 1;
    	    VBars[i][j].min = yVal - 5;
    	    VBars[i][j].max = yVal + 5;
    	}
    }
    
    for (i=0; i<NUM_CURVES/2; i++) {
    	Curves[i].vertBars = showErrorBars ? VBars[i] : NULL;
    	Curves[i].horizBars = showErrorBars ? HBars[i] : NULL;
    	Curves[i].lineStyle = XY_NO_LINE;
    }
    
    XYSetContents(XYPlot, Curves, NUM_CURVES,
    	    init ? XY_RESCALE : XY_RESCALE_AT_MAX);
}

static void createHistograms(Boolean init, Boolean showErrorBars)
{
    Pixel black = BlackPixelOfScreen(XtScreen(XYPlot));
    int i, j;
    float yVal;
    char name[100];

    for (i=0; i<NUM_HISTOGRAMS; i++) {
    	Histograms[i].linePixel = black;
    	Histograms[i].fillPixel = black;
    	sprintf(name, "Histogram%d", i);
    	Histograms[i].name = XmStringCreateSimple(name);
    }
    
    srand(100);
    for (i=0; i<NUM_HISTOGRAMS; i++) {
    	for (j=0; j<NUM_BINS; j++) {
    	    Histograms[i].bins[j] = rand() / (RAND_MAX / 100) - 30;
    	    HistErrBars[i][j].min = 1;
    	    HistErrBars[i][j].max = 1;
    	}
    }
    
    for (i=0; i<NUM_HISTOGRAMS/2; i++) {
    	Histograms[i].errorBars = showErrorBars ? HistErrBars[i] : NULL;
    }
    
    XYSetHistograms(XYPlot, Histograms, NUM_HISTOGRAMS,
    	    init ? XY_RESCALE : XY_RESCALE_AT_MAX, True);
}


static void zoomCB(Widget w, XtPointer data, caddr_t call_data)
{
    /* This one callback is used for all 3 zoom pushbuttons:
       Zoom In, Zoom Out, and Reset Zoom. */
       
    switch ((int)data) {
        case 1 : XYZoomIn(XYPlot); 
                 break;
        case 2 : XYZoomOut(XYPlot);
                 break;
        case 3 : XYResetZoom(XYPlot); 
                 break;         
    } 
}

static void btn2CB(Widget w, XtPointer data, XYCallbackStruct *callData)
{
    double x, y;
    
    x = XYWindowToDataX(callData->xform, callData->event->xbutton.x);
    y = XYWindowToDataY(callData->xform, callData->event->xbutton.y);
       
    printf("X %g, Y %g\n", x, y);
    printf("win X %g (orig %d), win Y %g (orig %d)\n",
    	    XYDataToWindowX(callData->xform, x), callData->event->xbutton.x,
    	    XYDataToWindowY(callData->xform, y), callData->event->xbutton.y);
}

static void btn3CB(Widget w, XtPointer data, XYCallbackStruct *callData)
{
    char charCodeString[2], fillCodeText[DF_MAX_PROMPT_LENGTH];
    unsigned int fillCode;
    int nRead, response;
    
    fillCodeText[0] = '\0';
    response = DialogF(DF_PROMPT, w, 2,
    	    "Enter fill pattern (hex)", fillCodeText, "OK", "Cancel");
    if (response == 2)
    	return;
    nRead = sscanf(fillCodeText, "%x", &fillCode);
    if (nRead != 1) {
    	XBell(XtDisplay(w), 0);
	return;
    }
    Histograms[0].fillStyle = fillCode;
    XYUpdateHistogramStyles(w, Histograms, True);

}

static void printCB(Widget w, caddr_t client_d, caddr_t call_d)
{
    /* This routine prints a PostScript version of the plot.
       XYPrintContents writes a post script file that is passed to
       the utility routine PrintFile, which presents a dialog and
       queues the file for printing. */
       
    static char fileName[] = "testXY.ps";
	
    XYPrintContents(XYPlot, fileName); 
#ifdef VMS
    PrintFile(w, fileName, fileName, True);
#else
    PrintFile(w, fileName, fileName); 
#endif /*VMS*/
    /* remove(fileName); */
}    
  
static void exitCB(Widget w, caddr_t client_data, caddr_t call_data)
{ 
    exit(0);
}
